unit Textures;

interface

uses
  Windows, dglOpenGL, Graphics, Classes, JPEG, SysUtils, Dialogs;

var
 LastFormat   : Cardinal;
 LastID       : Cardinal;
 Target       : Cardinal;
 LastBinding  : Cardinal;
 LastWidth    : Word;
 LastHeight   : Word;
 LastAlpha    : Boolean;

function LoadTexture(Filename: String; var Texture: TGLuint): Boolean;

implementation

{------------------------------------------------------------------}
{  Swap bitmap format from BGR to RGB                              }
{------------------------------------------------------------------}
procedure SwapRGB(data : Pointer; Size : Integer);
asm
  mov ebx, eax
  mov ecx, size

@@loop :
  mov al,[ebx+0]
  mov ah,[ebx+2]
  mov [ebx+2],al
  mov [ebx+0],ah
  add ebx,3
  dec ecx
  jnz @@loop
end;


{------------------------------------------------------------------}
{  Create the Texture                                              }
{------------------------------------------------------------------}
function CreateTexture(Width, Height, Format : Word; pData : Pointer) : Integer;
var
  Texture : TGLuint;
begin
glDisable(GL_TEXTURE_1D);
glDisable(GL_TEXTURE_2D);
glDeleteTextures(1, @Texture);
glGenTextures(1, @Texture);
LastFormat  := Format;
LastWidth   := Width;
LastHeight  := Height;
LastID      := Texture;
LastAlpha   := False;
if (Format = GL_RGBA) or (Format = GL_RGBA8) then
 LastAlpha := True;
// 2D-Textur
if Height > 1 then
 begin
 glEnable(GL_TEXTURE_2D);
 glBindTexture(Target, Texture);
 glTexParameteri(Target, GL_TEXTURE_MAG_FILTER, GL_LINEAR); { only first two can be used }
 glTexParameteri(Target, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR); { all of the above can be used }
 if Format = GL_RGBA then
   gluBuild2DMipmaps(Target, GL_RGBA8, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, pData)
 else
   gluBuild2DMipmaps(Target, GL_RGB8, Width, Height, GL_RGB, GL_UNSIGNED_BYTE, pData);
 LastBinding := Target;
 end;
// 1D-Textur
if Height = 1 then
 begin
 glEnable(GL_TEXTURE_1D);
 glBindTexture(GL_TEXTURE_1D, Texture);
 gluBuild1DMipmaps(GL_TEXTURE_1D, Format, Width, Format, GL_UNSIGNED_BYTE, pData);
 //glTexImage1D(GL_TEXTURE_1D, 0, GL_RGBA, Width, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, pData);
 glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
 glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 LastBinding := GL_TEXTURE_1D;
 end;
Result := Texture;
end;


{------------------------------------------------------------------}
{  Load BMP textures                                               }
{------------------------------------------------------------------}
function LoadBMPTexture(Filename: String; var Texture : TGLuint; LoadFromResource : Boolean) : Boolean;
var
  FileHeader: BITMAPFILEHEADER;
  InfoHeader: BITMAPINFOHEADER;
  Palette: array of RGBQUAD;
  BitmapFile: THandle;
  BitmapLength: LongWord;
  PaletteLength: LongWord;
  ReadBytes: LongWord;
  Width, Height : Integer;
  pData : Pointer;

  // used for loading from resource
  ResStream : TResourceStream;
begin
  result :=FALSE;

  if LoadFromResource then // Load from resource
  begin
    try
      ResStream := TResourceStream.Create(hInstance, PChar(copy(Filename, 1, Pos('.', Filename)-1)), 'BMP');
      ResStream.ReadBuffer(FileHeader, SizeOf(FileHeader));  // FileHeader
      ResStream.ReadBuffer(InfoHeader, SizeOf(InfoHeader));  // InfoHeader
      PaletteLength := InfoHeader.biClrUsed;
      SetLength(Palette, PaletteLength);
      ResStream.ReadBuffer(Palette, PaletteLength);          // Palette

      Width := InfoHeader.biWidth;
      Height := InfoHeader.biHeight;

      BitmapLength := InfoHeader.biSizeImage;
      if BitmapLength = 0 then
        BitmapLength := Width * Height * InfoHeader.biBitCount Div 8;

      GetMem(pData, BitmapLength);
      ResStream.ReadBuffer(pData^, BitmapLength);            // Bitmap Data
      ResStream.Free;
    except on
      EResNotFound do
      begin
        MessageBox(0, PChar('File not found in resource - ' + Filename), PChar('BMP Texture'), MB_OK);
        Exit;
      end
      else
      begin
        MessageBox(0, PChar('Unable to read from resource - ' + Filename), PChar('BMP Unit'), MB_OK);
        Exit;
      end;
    end;
  end
  else
  begin   // Load image from file
    BitmapFile := CreateFile(PChar(Filename), GENERIC_READ, FILE_SHARE_READ, nil, OPEN_EXISTING, 0, 0);
    if (BitmapFile = INVALID_HANDLE_VALUE) then begin
      MessageBox(0, PChar('Error opening ' + Filename), PChar('BMP Unit'), MB_OK);
      Exit;
    end;

    // Get header information
    ReadFile(BitmapFile, FileHeader, SizeOf(FileHeader), ReadBytes, nil);
    ReadFile(BitmapFile, InfoHeader, SizeOf(InfoHeader), ReadBytes, nil);

    // Get palette
    PaletteLength := InfoHeader.biClrUsed;
    SetLength(Palette, PaletteLength);
    ReadFile(BitmapFile, Palette, PaletteLength, ReadBytes, nil);
    if (ReadBytes <> PaletteLength) then begin
      MessageBox(0, PChar('Error reading palette'), PChar('BMP Unit'), MB_OK);
      Exit;
    end;

    Width  := InfoHeader.biWidth;
    Height := InfoHeader.biHeight;
    BitmapLength := InfoHeader.biSizeImage;
    if BitmapLength = 0 then
      BitmapLength := Width * Height * InfoHeader.biBitCount Div 8;

    // Get the actual pixel data
    GetMem(pData, BitmapLength);
    ReadFile(BitmapFile, pData^, BitmapLength, ReadBytes, nil);
    if (ReadBytes <> BitmapLength) then begin
      MessageBox(0, PChar('Error reading bitmap data'), PChar('BMP Unit'), MB_OK);
      Exit;
    end;
    CloseHandle(BitmapFile);
  end;

  // Bitmaps are stored BGR and not RGB, so swap the R and B bytes.
  SwapRGB(pData, Width*Height);

  Texture :=CreateTexture(Width, Height, GL_RGB, pData);
  FreeMem(pData);
  result :=TRUE;
end;

// =============================================================================
//  LoadJPGTexture
// =============================================================================
function LoadJPGTexture(Filename: String; var Texture: TGLuint; LoadFromResource : Boolean): Boolean;
var
 Data      : array of LongWord;
 W, Width  : Integer;
 H, Height : Integer;
 BMP       : TBitmap;
 JPG       : TJPEGImage;
 C         : LongWord;
 Line      : ^LongWord;
 ResStream : TResourceStream;      // used for loading from resource
begin
Result := False;
JPG    := TJPEGImage.Create;
if LoadFromResource then
 begin
 try
  ResStream := TResourceStream.Create(hInstance, PChar(copy(Filename, 1, Pos('.', Filename)-1)), 'JPEG');
  JPG.LoadFromStream(ResStream);
  ResStream.Free;
 except
  on EResNotFound do
   begin
   MessageBox(0, PChar('File not found in resource - ' + Filename), PChar('JPG Texture'), MB_OK);
   exit;
   end
  else
   begin
   MessageBox(0, PChar('Couldn''t load JPG Resource - "'+ Filename +'"'), PChar('BMP Unit'), MB_OK);
   exit;
   end;
 end;
 end
else
 begin
 try
  JPG.LoadFromFile(Filename);
  except
   MessageBox(0, PChar('Couldn''t load JPG - "'+ Filename +'"'), PChar('BMP Unit'), MB_OK);
   exit;
   end;
 end;
// Create Bitmap
BMP := TBitmap.Create;
BMP.pixelformat := pf32bit;
BMP.width       := JPG.width;
BMP.height      := JPG.height;
BMP.canvas.draw(0,0,JPG);
Width  := BMP.Width;
Height := BMP.Height;
SetLength(Data, Width*Height);
for h := 0 to Height-1 do
 begin
 Line := BMP.scanline[Height-H-1];   // flip JPEG
 for w :=0 to Width-1 do
  begin
  c := Line^ and $FFFFFF; // Need to do a color swap
  Data[W+(H*Width)] :=(((c and $FF) shl 16)+(c shr 16)+(c and $FF00)) or $FF000000;  // 4 channel.
  inc(Line);
  end;
 end;
BMP.Free;
JPG.Free;
Texture    := CreateTexture(Width, Height, GL_RGBA, addr(Data[0]));
LastFormat := GL_RGB;
result     := True;
SetLength(Data, 0);
end;


{------------------------------------------------------------------}
{  Loads 24 and 32bpp (alpha channel) TGA textures                 }
{------------------------------------------------------------------}
function LoadTGATexture(Filename: String; var Texture: TGLuint; LoadFromResource : Boolean): Boolean;
var
  TGAHeader : packed record   // Header type for TGA images
    FileType     : Byte;
    ColorMapType : Byte;
    ImageType    : Byte;
    ColorMapSpec : Array[0..4] of Byte;
    OrigX  : Array [0..1] of Byte;
    OrigY  : Array [0..1] of Byte;
    Width  : Array [0..1] of Byte;
    Height : Array [0..1] of Byte;
    BPP    : Byte;
    ImageInfo : Byte;
  end;
  TGAFile   : File;
  bytesRead : Integer;
  image     : Pointer;    {or PRGBTRIPLE}
  CompImage : Pointer;
  Width, Height : Integer;
  ColorDepth    : Integer;
  ImageSize     : Integer;
  BufferIndex : Integer;
  currentByte : Integer;
  CurrentPixel : Integer;
  I : Integer;
  Front: ^Byte;
  Back: ^Byte;
  Temp: Byte;

  ResStream : TResourceStream;      // used for loading from resource

  // Copy a pixel from source to dest and Swap the RGB color values
  procedure CopySwapPixel(const Source, Destination : Pointer);
  asm
    push ebx
    mov bl,[eax+0]
    mov bh,[eax+1]
    mov [edx+2],bl
    mov [edx+1],bh
    mov bl,[eax+2]
    mov bh,[eax+3]
    mov [edx+0],bl
    mov [edx+3],bh
    pop ebx
  end;

begin
Result    := False;
ResStream := NIL;
CompImage := NIL;
Image     := NIL;
// Load from resource
if LoadFromResource then
 begin
 try
  ResStream := TResourceStream.Create(hInstance, PChar(copy(Filename, 1, Pos('.', Filename)-1)), 'TGA');
  ResStream.ReadBuffer(TGAHeader, SizeOf(TGAHeader));
  result := True;
 except
  on EResNotFound do
   begin
   MessageBox(0, PChar('File not found in resource - ' + Filename), PChar('TGA Texture'), MB_OK);
   exit;
   end
  else
   begin
   MessageBox(0, PChar('Unable to read from resource - ' + Filename), PChar('BMP Unit'), MB_OK);
   exit;
   end;
 end;
 end
else
 begin
 if FileExists(Filename) then
  begin
  AssignFile(TGAFile, Filename);
  Reset(TGAFile, 1);
  // Read in the bitmap file header
  BlockRead(TGAFile, TGAHeader, SizeOf(TGAHeader));
  Result := TRUE;
  end
 else
  begin
  MessageBox(0, PChar('File not found  - ' + Filename), PChar('TGA Texture'), MB_OK);
  exit;
  end;
 end;
if Result then
 begin
 Result := False;
 // Only support 24, 32 bit images { TGA_RGB } + { Compressed RGB }
 if (TGAHeader.ImageType <> 2) and (TGAHeader.ImageType <> 10) then
  begin
  Result := False;
  CloseFile(tgaFile);
  MessageBox(0, PChar('Couldn''t load "'+ Filename +'". Only 24 and 32bit TGA supported.'), PChar('TGA File Error'), MB_OK);
  exit;
  end;
 // Don't support colormapped files
 if TGAHeader.ColorMapType <> 0 then
  begin
  Result := False;
  CloseFile(TGAFile);
  MessageBox(0, PChar('Couldn''t load "'+ Filename +'". Colormapped TGA files not supported.'), PChar('TGA File Error'), MB_OK);
  exit;
  end;
 // Get the width, height, and color depth
 Width      := TGAHeader.Width[0]  + TGAHeader.Width[1]  * 256;
 Height     := TGAHeader.Height[0] + TGAHeader.Height[1] * 256;
 ColorDepth := TGAHeader.BPP;
 ImageSize  := Width*Height*(ColorDepth div 8);
 if ColorDepth < 24 then
  begin
  Result := False;
  CloseFile(TGAFile);
  MessageBox(0, PChar('Couldn''t load "'+ Filename +'". Only 24 and 32 bit TGA files supported.'), PChar('TGA File Error'), MB_OK);
  exit;
  end;
 GetMem(Image, ImageSize);
 // Standard 24, 32 bit TGA file
 if TGAHeader.ImageType = 2 then
  begin
  // Load from resource
  if LoadFromResource then
   begin
   try
    ResStream.ReadBuffer(Image^, ImageSize);
    ResStream.Free;
   except
    MessageBox(0, PChar('Unable to read from resource - ' + Filename), PChar('BMP Unit'), MB_OK);
    exit;
   end;
   end
  else
   // Read in the image from file
   begin
   BlockRead(TGAFile, image^, ImageSize, bytesRead);
   if bytesRead <> ImageSize then
    begin
    Result := False;
    FreeMem(Image);
    CloseFile(TGAFile);
    MessageBox(0, PChar('Couldn''t read file "'+ Filename +'".'), PChar('TGA File Error'), MB_OK);
    exit;
    end
   end;
  // TGAs are stored BGR and not RGB, so swap the R and B bytes.
  // 32 bit TGA files have alpha channel and gets loaded differently
  if TGAHeader.BPP = 24 then
   begin
   for i := 0 to Width * Height - 1 do
    begin
    Front  := Pointer(Integer(Image) + I*3);
    Back   := Pointer(Integer(Image) + I*3 + 2);
    Temp   := Front^;
    Front^ := Back^;
    Back^  := Temp;
    end;
   Texture := CreateTexture(Width, Height, GL_RGB, Image);
   end
  else
   begin
   for i := 0 to Width * Height - 1 do
    begin
    Front  := Pointer(Integer(Image) + I*4);
    Back   := Pointer(Integer(Image) + I*4 + 2);
    Temp   := Front^;
    Front^ := Back^;
    Back^  := Temp;
    end;
   Texture := CreateTexture(Width, Height, GL_RGBA, Image);
   end;
 end;
 // Compressed 24, 32 bit TGA files
 if TGAHeader.ImageType = 10 then
  begin
  ColorDepth   := ColorDepth DIV 8;
  CurrentByte  := 0;
  CurrentPixel := 0;
  BufferIndex  := 0;
  if LoadFromResource then
   begin
   try
    GetMem(CompImage, ResStream.Size-sizeOf(TGAHeader));
    ResStream.ReadBuffer(CompImage^, ResStream.Size-sizeOf(TGAHeader));   // load compressed date into memory
    ResStream.Free;
   except
    MessageBox(0, PChar('Unable to read from resource - ' + Filename), PChar('BMP Unit'), MB_OK);
    exit;
   end;
   end
  else
   begin
   GetMem(CompImage, FileSize(TGAFile)-sizeOf(TGAHeader));
   BlockRead(TGAFile, CompImage^, FileSize(TGAFile)-sizeOf(TGAHeader), BytesRead);   // load compressed data into memory
   if bytesRead <> FileSize(TGAFile)-sizeOf(TGAHeader) then
    begin
    Result := False;
    FreeMem(CompImage);
    CloseFile(TGAFile);
    MessageBox(0, PChar('Couldn''t read file "'+ Filename +'".'), PChar('TGA File Error'), MB_OK);
    exit;
    end
   end;
  // Extract pixel information from compressed data
  repeat
  Front := Pointer(Integer(CompImage) + BufferIndex);
  inc(BufferIndex);
  if Front^ < 128 then
   begin
   for i := 0 to Front^ do
    begin
    CopySwapPixel(Pointer(Integer(CompImage)+BufferIndex+I*ColorDepth), Pointer(Integer(image)+CurrentByte));
    CurrentByte := CurrentByte + ColorDepth;
    inc(CurrentPixel);
    end;
   BufferIndex := BufferIndex + (Front^+1)*ColorDepth
   end
  else
   begin
   for i := 0 to Front^ -128 do
    begin
    CopySwapPixel(Pointer(Integer(CompImage)+BufferIndex), Pointer(Integer(image)+CurrentByte));
    CurrentByte := CurrentByte + ColorDepth;
    inc(CurrentPixel);
    end;
   BufferIndex := BufferIndex + ColorDepth
   end;
  until CurrentPixel >= Width*Height;
  if ColorDepth = 3 then
   Texture := CreateTexture(Width, Height, GL_RGB, Image)
  else
   Texture := CreateTexture(Width, Height, GL_RGBA, Image);
  end;
  Result := TRUE;
  CloseFile(TGAFile);
  end;
if Assigned(Image) then
 FreeMem(Image);
if Assigned(CompImage) then
 FreeMem(CompImage);
end;


{------------------------------------------------------------------}
{  Determines file type and sends to correct function              }
{------------------------------------------------------------------}
function LoadTexture(Filename: String; var Texture : TGLuint) : Boolean;
begin
Result := False;
if copy(Uppercase(filename), length(filename)-3, 4) = '.BMP' then
 Result := LoadBMPTexture(Filename, Texture, False);
if copy(Uppercase(filename), length(filename)-3, 4) = '.JPG' then
 Result := LoadJPGTexture(Filename, Texture, False);
if copy(Uppercase(filename), length(filename)-3, 4) = '.TGA' then
 Result := LoadTGATexture(Filename, Texture, False);
end;

initialization
 Target := GL_TEXTURE_2D;

end.