unit glLoadMD5;

interface

uses
 Windows,
 SysUtils,
 Classes,


 dglOpenGL;

type
 TBone = record
   BindPos : array[0..3] of Single;
   BindMat : array[0..8] of Single;
   Name    : String;
   Parent  : String;
  end;
 TSubMesh = record
   Shader   : String;
   Vertex   : array of record
               U,V        : Single;
               BlendIndex : Integer;
               BlendCount : Integer;
              end;
   Triangle : array[0..2] of Integer;
   Weight   : record
               Bone : Integer;
               Bias : Single;
               X    : Single;
               Y    : Single;
               Z    : Single;
              end;
  end;
 TMeshMD5 = object
   SubMesh : array of TSubMesh;
   Bone    : array of TBone;
   procedure LoadFromFile(pFileName : String);
  end;

var
 MeshMD5 : TMeshMD5;
 Log     : TStringList;

implementation

function GetToken(pStartToken, pLine : String;pTokenNumber : Integer) : String;
var
 TokenList : array of String;
 i,start   : Integer;
begin
start := Pos(pStartToken, pLine)+Length(pStartToken);
SetLength(TokenList, 1);
for i := start to Length(pLine) do
 case pLine[i] of
  Chr(VK_SPACE) : if TokenList[High(TokenList)] <> '' then
                   SetLength(TokenList, Length(TokenList)+1);
  Chr(VK_TAB)   : if TokenList[High(TokenList)] <> '' then
                   SetLength(TokenList, Length(TokenList)+1);
 else
  TokenList[High(TokenList)] := TokenList[High(TokenList)]+pLine[i];
 end;
if pTokenNumber <= Length(TokenList) then
 Result := TokenList[pTokenNumber];
end;

procedure TMeshMD5.LoadFromFile(pFileName : String);
var
 F : TextFile;
 S : String;
 n : Integer;
 i : Integer;
begin
Log := TStringList.Create;
DecimalSeparator := '.';
AssignFile(F, pFileName);
Reset(F);
// ==== Bones ==================================================================
// Find number of bones
repeat
ReadLn(F, S);
if Pos('numbones', S) > 0 then
 begin
 SetLength(Bone, StrToInt(Copy(S, Length('numbones')+2, Length(S)-Length('numbones'))));
 break;
 end;
until EOF(F);
if EOF(F) then halt(0);
// Find bones
for n := 0 to High(Bone) do
 begin
 Reset(F);
 repeat
 ReadLn(F, S);
 if Pos('bone '+IntToStr(n), S) > 0 then
  begin
  // Read bone
  ReadLn(F, S);
  Bone[n].Name := Copy(GetToken('name', S, 0), 2, Length(GetToken('name', S, 0))-2);
  ReadLn(F, S);
  Bone[n].BindPos[0] := StrToFloat(GetToken('bindpos', S, 0));
  Bone[n].BindPos[1] := StrToFloat(GetToken('bindpos', S, 1));
  Bone[n].BindPos[2] := StrToFloat(GetToken('bindpos', S, 2));
  ReadLn(F, S);
  Bone[n].BindMat[0] := StrToFloat(GetToken('bindmat', S, 0));
  Bone[n].BindMat[1] := StrToFloat(GetToken('bindmat', S, 1));
  Bone[n].BindMat[2] := StrToFloat(GetToken('bindmat', S, 2));
  Bone[n].BindMat[3] := StrToFloat(GetToken('bindmat', S, 3));
  Bone[n].BindMat[4] := StrToFloat(GetToken('bindmat', S, 4));
  Bone[n].BindMat[5] := StrToFloat(GetToken('bindmat', S, 5));
  Bone[n].BindMat[6] := StrToFloat(GetToken('bindmat', S, 6));
  Bone[n].BindMat[7] := StrToFloat(GetToken('bindmat', S, 7));
  Bone[n].BindMat[8] := StrToFloat(GetToken('bindmat', S, 8));
  ReadLn(F, S);
  if S = '}' then
   Bone[n].Parent := ''
  else
   Bone[n].Parent := Copy(GetToken('parent', S, 0), 2, Length(GetToken('parent', S, 0))-2);
  Log.Add('Bone '+IntToStr(n)+' loaded');
  Log.SaveToFile('log.txt');
  break;
  end;
 until EOF(F);
 if EOF(F) then halt(0);
 end;
// ==== Meshes =================================================================
Reset(F);
// Find number of meshes
repeat
ReadLn(F, S);
if Pos('nummeshes', S) > 0 then
 begin
 SetLength(SubMesh, StrToInt(Copy(S, Length('nummeshes')+2, Length(S)-Length('nummeshes'))));
 break;
 end;
until EOF(F);
if EOF(F) then halt(0);
// Find meshes
for n := 0 to High(SubMesh) do
 begin
 Reset(F);
 repeat
 ReadLn(F, S);
 if Pos('mesh '+IntToStr(n), S) > 0 then
  begin
  ReadLn(F,S);
  SubMesh[n].Shader := GetToken('shader', S, 0);
  ReadLn(F,S);
  ReadLn(F,S);
  SetLength(SubMesh[n].Vertex, StrToInt(GetToken('numverts', S, 0)));
  // Read in vertices
  for i := 0 to High(SubMesh[n].Vertex) do
   begin
   SubMesh[n].Vertex[i].U := 
   end;
  Log.Add('Mesh '+IntToStr(n)+' loaded');
  Log.SaveToFile('log.txt');
  break;
  end;
 until EOF(F);
 if EOF(F) then halt(0);
 end;
 
CloseFile(F);
Log.Free;
end;

end.
