// =============================================================================
//
//   Delphi 3DS-Demo of the Newton Physics Engine
//    Newton Game Dynamics  2003-2004 by Julio Jerez
//     http://physicsengine.com
//    Headerport for Delphi by S.Spasov
//     http://www.geocities.com/delphinewton/
//
// =============================================================================
//   Copyright  2003-2004 by Sascha Willems - http://www.delphigl.de
//   visit the Delphi OpenGL Community - http://www.delphigl.com
// =============================================================================
//   Contents of this file are subject to the GNU Public License (GPL) which can
//   be obtained here : http://opensource.org/licenses/gpl-license.php
//   So only use this file if you fully unterstand that license!!!
// =============================================================================
//   Short description :
//    This demo shows how to use a world build in 3D Studio (or any other 3D
//    modeller that can export to 3DS) with newton. You can move a ball around
//    in that level, kind of like the game "marble madness".
// =============================================================================

unit glNewtonScene3DS_Mainform;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  Math,
  ExtCtrls,

  NewtonImport,
  Textures,
  gl3DS,
  glMath,
  glMatrixHelper,
  dglOpenGL;

type
  TGLForm = class(TForm)
    Open3DSDialog: TOpenDialog;
    InitTimer: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
    procedure InitTimerTimer(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    RC        : HGLRC;
    DC        : HDC;
    ShowFPS   : Boolean;
    FontBase  : GLUInt;
    StartTick : Cardinal;
    Frames    : Integer;
    FPS       : Single;
    procedure BuildFont(pFontName : String);
    procedure PrintText(pX,pY : Integer; const pText : String);
    procedure ShowText;
    procedure Render;
  end;

  TRagDollBone = class;
  TRagDollBone = class
    Body       : PNewtonBody;
    Size       : TVector3f;
    Matrix     : TMatrix4f;
    Color      : TVector3f;
    ID         : Integer;
    SelExclude : Boolean;
    Texture    : PglUInt;
    Name       : String;
    Children   : array of TRagDollBone;
    constructor Create;
    procedure Render;
   end;

  TRagDoll = class
   private
    procedure BuildRagDoll(pRagDoll : PNewtonRagDoll; pParentDollBone : PNewtonRagDollBone; pVelocity : TVector3f; var pBone : TRagDollBone);
   public
    RagDoll          : PNewtonRagDoll;
    spine0_Bone      : TRagDollBone;
    spine1_Bone      : TRagDollBone;
    head_Bone        : TRagDollBone;
    rUpperArm_Bone   : TRagDollBone;
    lUpperArm_Bone   : TRagDollBone;
    rLowerArm_Bone   : TRagDollBone;
    lLowerArm_Bone   : TRagDollBone;
    rUpperLeg_Bone   : TRagDollBone;
    rLowerLeg_Bone   : TRagDollBone;
    lUpperLeg_Bone   : TRagDollBone;
    lLowerLeg_Bone   : TRagDollBone;
    Bone             : array of TRagDollBone;
    constructor Create(pPosition, pVelocity : TVector3f; pHangByHead : Boolean = False);
    destructor Destroy; override;
    procedure Render;
   end;

const
 LightPos      : array[0..3] of Single = (0, 60, 0, 1);
 FOV           = 60;
 zNear         = 1;
 zFar          = 512;
 PIOver180     = PI/180;
 GLBoxData : array[0..143] of Single =
  (-1,1,1, 1,1,1, 1,-1,1, -1,-1,1,
   -1,1,-1, 1,1,-1, 1,-1,-1, -1,-1,-1,
   -1,1,1, -1,1,-1, -1,-1,-1, -1,-1,1,
   1,1,1, 1,1,-1, 1,-1,-1, 1,-1,1,
   -1,1,1, -1,1,-1, 1,1,-1, 1,1,1,
   -1,-1,1, -1,-1,-1, 1,-1,-1, 1,-1,1,
   0,0,1,0,0,1,0,0,1,0,0,1,0,0,-1,0,0,-1,0,0,-1,0,0,-1,
   -1,0,0,-1,0,0,-1,0,0,-1,0,0,1,0,0,1,0,0,1,0,0,1,0,0,
   0,1,0,0,1,0,0,1,0, 0,1,0,0,-1,0,0,-1,0,0,-1,0,0,-1,0);
 GLBoxTC : array[0..47] of Single =
  (0,1, 1,1, 1,0, 0,0, 0,1, 1,1, 1,0, 0,0,
   1,1, 1,0, 0,0, 0,1, 1,1, 1,0, 0,0, 0,1,
   0,1, 0,0, 1,0, 1,1, 0,1, 0,0, 1,0, 1,1);
 NewtonGravity = -9.8;

var
  GLForm             : TGLForm;
  File3DS            : String;
  Scene3DS           : TAll3DSMesh;
  CollTree3DS        : PNewtonCollision;
  Body3DS            : PNewtonBody;
  Collider           : PNewtonCollision;
  Init               : Boolean;
  NewtonWorld        : PNewtonWorld;
  RagDoll      : array of TRagDoll;
  Q                  : PgluQuadric;
  QPCFreq            : Int64;
  CanJump            : Boolean = True;

  BallBody           : PNewtonBody;
  BallTorque         : TVector3f;
  BallMatrix         : TMatrix4f;

  cameraTorqueVector : TVector3f;
  cameraDir          : TVector3f;
  cameraEyepoint     : TVector3f;
  cameraDirMat       : TMatrix4f;
  Mouse0             : TPoint;
  CurrStartPoint     : Integer;
  UserData           : Pointer;

  yawAngle           : Single;
  rollAngle          : Single;
  Zoom               : Single;
  // Materials
  MatWorld           : Integer;
  MatSphere          : Integer;
  // Texture IDs
  SphereTexID        : glUInt;
  SkyTexID           : glUInt;
  LoadTexID          : glUInt;
  NStripTexID        : glUInt;
  // Display list
  SphereListID       : glUInt;
  SkySphereListID    : glUInt;

implementation

{$R *.dfm}

// =============================================================================
//  Create3DSCollisionTree
// =============================================================================
//  Sends the data of a 3DS scene to newton to let it build a static
//  tree collision.
// =============================================================================
procedure Create3DSCollisionTree(const pScene3DS : TAll3DSMesh);
var
 TmpFace : array[0..2] of T3Dpoint;
 TmpM    : TMatrix4f;
 Min     : TVector3f;
 Max     : TVector3f;
 m,f     : Integer;
begin
CollTree3DS := NewtonCreateTreeCollision(NewtonWorld, nil);
// Go through all meshes and add the faces of it to the tree collision
NewtonTreeCollisionBeginBuild(CollTree3DS);
if pScene3DS.NumMeshes > 0 then
 for m := 0 to pScene3DS.NumMeshes-1 do
  if Pos('start', pScene3DS.Mesh[m].Name) = 0 then // I use some predefined startpoints in the demo, they are excluded from collision
   if pScene3DS.Mesh[m].NumFaces > 0 then
    with pScene3DS.Mesh[m] do
     begin
     f := 0;
     repeat
     TmpFace[0] := Vertex[Face[f+2]];
     TmpFace[1] := Vertex[Face[f+1]];
     TmpFace[2] := Vertex[Face[f]];
     NewtonTreeCollisionAddFace(CollTree3DS, 3, @TmpFace[0], SizeOf(T3DPoint), 1);
     inc(f,3);
     until f > NumFaces-1;
     end;
NewtonTreeCollisionEndBuild(CollTree3DS, 0);
Body3DS := NewtonCreateBody(NewtonWorld, CollTree3DS);
// Set position to 0/0/0
Matrix_SetIdentity(TmpM);
NewtonBodySetMatrix(Body3DS, @TmpM[0,0]);
// Set material
NewtonBodySetMaterialGroupID(Body3DS, MatWorld);
// Get AABB and set limits of the newton world
NewtonCollisionCalculateAABB(CollTree3DS, @TmpM[0,0], @Min[0], @Max[0]);
NewtonSetWorldSize(NewtonWorld, @Min[0], @Max[0]);
NewtonReleaseCollision(NewtonWorld, CollTree3DS);
end;

// =============================================================================
//  Newton Callbacks
// =============================================================================

// =============================================================================
//  BallCharacterApplyForceAndTorque
// =============================================================================
procedure BallCharacterApplyForceAndTorque(const Body : PNewtonBody); cdecl;
var
 mass       : Single;
 Ixx        : Single;
 Iyy        : Single;
 Izz        : Single;
 Force      : TVector3f;
 Torque     : TVector3f;
 TmpM       : TMatrix4f;
 LateralDir : TVector3f;
 Omega      : TVector3f;
 TorqueDir  : TVector3f;
 TmpS       : Single;
 TmpV       : TVector3f;
begin
NewtonBodyGetMassMatrix(Body, @mass, @Ixx, @Iyy, @Izz);
Force := V3(0, -Mass * 19.8, 0);
NewtonBodySetForce(Body, @Force[0]);
// calculate a rotation matrix from the camera heading
Matrix_SetIdentity(TmpM);
LateralDir := VCross(CameraDir, V3(0,1,0));
TmpS       := LateralDir[0]*LateralDir[0] + LateralDir[1]*LateralDir[1] + LateralDir[2]*LateralDir[2];
TmpM[2,0]  := LateralDir[0] * 1/TmpS;
TmpM[2,1]  := LateralDir[1] * 1/TmpS;
TmpM[2,2]  := LateralDir[2] * 1/TmpS;
TmpV := VCross(V3(TmpM[1,0],TmpM[1,1],TmpM[1,2]), V3(TmpM[2,0],TmpM[2,1],TmpM[2,2]));
TmpM[0,0]  := TmpV[0];
TmpM[0,1]  := TmpV[1];
TmpM[0,2]  := TmpV[2];
// rotate the torque direction to align with the camera
TorqueDir := CameraTorqueVector;
Matrix_RotateVect(TmpM, TorqueDir);
NewtonBodyGetOmega(BallBody, @Omega[0]);
TmpS   := Omega[0]*TorqueDir[0] + Omega[1]*TorqueDir[1] + Omega[2]*TorqueDir[2];
Torque := V3(TorqueDir[0]*(Ixx*75) - TorqueDir[0]*(2*Ixx*TmpS),
             TorqueDir[1]*(Ixx*75) - TorqueDir[1]*(2*Ixx*TmpS),
             TorqueDir[2]*(Ixx*75) - TorqueDir[2]*(2*Ixx*TmpS));
NewtonBodySetTorque(BallBody, @Torque[0]);
end;

// =============================================================================
//  ApplyForceAndTorqueEvent
// =============================================================================
procedure ApplyForceAndTorqueEvent(const Body : PNewtonBody); cdecl;
var
 Mass   : Single;
 Ixx    : Single;
 Iyy    : Single;
 Izz    : Single;
 Force  : TVector3f;
 Torque : TVector3f;
begin
NewtonBodyGetMassMatrix(Body, @Mass, @Ixx, @Iyy, @Izz);
Force  := V3(0, NewtonGravity * Mass, 0);
Torque := V3(0, 0, 0);
NewtonBodyAddForce(Body, @force[0]);
NewtonBodyAddTorque(Body, @torque[0]);
end;

// =============================================================================
//  AddChildren
// =============================================================================
procedure AddChildren(var pParent : TRagDollBone; pChildren : TRagDollBone);
begin
SetLength(pParent.Children, Length(pParent.Children)+1);
pParent.Children[High(pParent.Children)] := pChildren;
end;

// =============================================================================
//  TRagDoll.Create
// =============================================================================

// =============================================================================
//  TRagDollBone.Create
// =============================================================================
constructor TRagDollBone.Create;
begin
Texture := nil;
end;

// =============================================================================
//  TRagDollBone.Render
// =============================================================================
procedure TRagDollBone.Render;
var
 Matrix : TMatrix4f;
begin           
if Body = nil then
 raise Exception.Create('Body = nil for '+Name);
NewtonBodyGetMatrix(Body, @Matrix);
if Texture <> nil then
 begin
 glEnable(GL_TEXTURE_2D);
 glBindTexture(GL_TEXTURE_2D, Texture^);
 end;
glPushMatrix;
 glColor3f(Color[0], Color[1], Color[2]);
 glMultMatrixf(@Matrix);
 glScalef(Size[0]/2, Size[1]/2, Size[2]/2);
 glVertexPointer(3, GL_FLOAT, 0, @GLBoxData[0]);
 glNormalPointer(GL_FLOAT, 0, @GLBoxData[72]);
 glTexCoordPointer(2, GL_FLOAT, 0, @GlBoxTC[0]);
 glDrawArrays(GL_QUADS, 0, 48);
 glDisable(GL_TEXTURE_2D);
 glColor3f(1,1,1);
glPopMatrix;
glDisable(GL_TEXTURE_2D);
end;

// =============================================================================
//  TRagDoll
// =============================================================================

// =============================================================================
//  TRagDoll.Create
// =============================================================================
constructor TRagDoll.Create(pPosition, pVelocity : TVector3f; pHangByHead : Boolean = False);
const
 HeadSize = 0.5;
var
 SpineSize : TVector3f;
 HeadBody  : PNewtonBody;
 HeadBone  : PNewtonRagDollBone;
 Mat       : TMatrix4f;
 Pivot     : TVector3f;
 MainColor : TVector3f;
 SkinColor : TVector3f;
begin
MainColor := V3(Random, Random, Random);
SkinColor := V3(0.95, 0.8, 0.95);
// set the lower spine bone (this is the root of the rag doll)
Spine0_Bone       := TRagDollBone.Create;
Spine0_Bone.Color := MainColor;
SpineSize         := V3(HeadSize*2, HeadSize, HeadSize*2);
Spine0_Bone.Size  := SpineSize;
Matrix_SetIdentity(Spine0_Bone.Matrix);
Matrix_SetTransform(Spine0_Bone.Matrix, V4(pPosition[0], pPosition[1], pPosition[2]));
Matrix_SetRotation(Spine0_Bone.Matrix, V3(90, 0, 90));
Spine0_Bone.ID := 0;
Spine0_Bone.Name := 'Spine0_Bone';

// set the upper spine bone
Spine1_Bone       := TRagDollBone.Create;
Spine1_Bone.Size  := SpineSize;
Spine1_Bone.Color := MainColor;
Matrix_SetIdentity(Spine1_Bone.Matrix);
Matrix_SetTransform(Spine1_Bone.Matrix, V4(SpineSize[0], 0, 0));
Matrix_SetRotation(Spine1_Bone.Matrix, V3(0, 0, 0));
Spine1_Bone.ID := 1;
Spine1_Bone.Name := 'Spine1_Bone';

// add the head
head_Bone       := TRagDollBone.Create;
head_Bone.Color := SkinColor;
head_Bone.Size  := V3(HeadSize, HeadSize, HeadSize);
Matrix_SetIdentity(head_Bone.Matrix);
Matrix_SetTransform(head_Bone.Matrix, V4(SpineSize[0], 0, 0));
Matrix_SetRotation(head_Bone.Matrix, V3(0, 0, 0));
head_Bone.ID := 2;
head_Bone.Name := 'head_Bone';

// Right Upper arm
rUpperArm_Bone       := TRagDollBone.Create;
rUpperArm_Bone.Color := MainColor;
rUpperArm_Bone.Size  := V3(HeadSize * 1.5, HeadSize * 0.5, HeadSize * 0.5);
Matrix_SetIdentity(rUpperArm_Bone.Matrix);
Matrix_SetTransform(rUpperArm_Bone.Matrix, V4(SpineSize[0], 0, SpineSize[0] * 0.55));
Matrix_SetRotation(rUpperArm_Bone.Matrix, V3(0, -90, 90));
rUpperArm_Bone.ID := 3;
rUpperArm_Bone.Name := 'rUpperArm_Bone';

// Right Lower arm
rLowerArm_Bone       := TRagDollBone.Create;
rLowerArm_Bone.Color := SkinColor;
rLowerArm_Bone.Size  := V3(HeadSize * 1.5, HeadSize * 0.5, HeadSize * 0.5);
Matrix_SetIdentity(rLowerArm_Bone.Matrix);
Matrix_SetTransform(rLowerArm_Bone.Matrix, V4(HeadSize * 1.5, 0, 0));
Matrix_SetRotation(rLowerArm_Bone.Matrix, V3(0, 0, 0));
rLowerArm_Bone.ID := 4;
rLowerArm_Bone.Name := 'rLowerArm_Bone';

// Left Upper arm
lUpperArm_Bone       := TRagDollBone.Create;
lUpperArm_Bone.Color := MainColor;
lUpperArm_Bone.Size  := V3(HeadSize * 1.5, HeadSize * 0.5, HeadSize * 0.5);
Matrix_SetIdentity(lUpperArm_Bone.Matrix);
Matrix_SetTransform(lUpperArm_Bone.Matrix, V4(SpineSize[0], 0, -SpineSize[0] * 0.55));
Matrix_SetRotation(lUpperArm_Bone.Matrix, V3(0, 90, 90));
lUpperArm_Bone.ID := 5;
lUpperArm_Bone.Name := 'lUpperArm_Bone';

// Left Lower arm
lLowerArm_Bone       := TRagDollBone.Create;
lLowerArm_Bone.Color := SkinColor;
lLowerArm_Bone.Size  := V3(HeadSize * 1.5, HeadSize * 0.5, HeadSize * 0.5);
Matrix_SetIdentity(lLowerArm_Bone.Matrix);
Matrix_SetTransform(lLowerArm_Bone.Matrix, V4(HeadSize * 1.5, 0, 0));
Matrix_SetRotation(lLowerArm_Bone.Matrix, V3(0, 0, 0));
lLowerArm_Bone.ID := 6;
lLowerArm_Bone.Name := 'lLowerArm_Bone';

// Right Upper leg
rUpperLeg_Bone       := TRagDollBone.Create;
rUpperLeg_Bone.Color := MainColor;
rUpperLeg_Bone.Size  := V3(HeadSize * 2.0, HeadSize * 0.75, HeadSize * 0.75);
Matrix_SetIdentity(rUpperleg_Bone.Matrix);
Matrix_SetTransform(rUpperleg_Bone.Matrix, V4(0, 0, SpineSize[0] * 0.35));
Matrix_SetRotation(rUpperleg_Bone.Matrix, V3(0, 0, 180));
rUpperleg_Bone.ID := 7;
rUpperleg_Bone.Name := 'rUpperleg_Bone';

// Right Lower leg
rLowerleg_Bone       := TRagDollBone.Create;
rLowerleg_Bone.Color := SkinColor;
rLowerleg_Bone.Size  := V3(HeadSize * 2.0, HeadSize * 0.65, HeadSize * 0.65);
Matrix_SetIdentity(rLowerleg_Bone.Matrix);
Matrix_SetTransform(rLowerleg_Bone.Matrix, V4(HeadSize * 2, 0, 0));
Matrix_SetRotation(rLowerleg_Bone.Matrix, V3(0, 0, 0));
rLowerleg_Bone.ID := 8;
rLowerleg_Bone.Name := 'rLowerleg_Bone';

// Left Upper leg
lUpperleg_Bone       := TRagDollBone.Create;
lUpperleg_Bone.Color := MainColor;
lUpperleg_Bone.Size  := V3(HeadSize * 2.0, HeadSize * 0.75, HeadSize * 0.75);
Matrix_SetIdentity(lUpperleg_Bone.Matrix);
Matrix_SetTransform(lUpperleg_Bone.Matrix, V4(0, 0, -SpineSize[0] * 0.35));
Matrix_SetRotation(lUpperleg_Bone.Matrix, V3(0, 0, 180));
lUpperleg_Bone.ID := 9;
lUpperleg_Bone.Name := 'lUpperleg_Bone';

// Left Lower leg
lLowerleg_Bone       := TRagDollBone.Create;
lLowerleg_Bone.Color := SkinColor;
lLowerleg_Bone.Size  := V3(HeadSize * 2.0, HeadSize * 0.65, HeadSize * 0.65);
Matrix_SetIdentity(lLowerleg_Bone.Matrix);
Matrix_SetTransform(lLowerleg_Bone.Matrix, V4(HeadSize * 2, 0, 0));
Matrix_SetRotation(lLowerleg_Bone.Matrix, V3(0, 0, 0));
lLowerleg_Bone.ID := 10;
lLowerleg_Bone.Name := 'lLowerleg_Bone';

// Build hierarchy
AddChildren(Spine0_Bone, Spine1_Bone);
AddChildren(Spine1_Bone, Head_Bone);
AddChildren(Spine1_Bone, rUpperArm_Bone);
AddChildren(rUpperArm_Bone, rLowerArm_Bone);
AddChildren(Spine1_Bone, lUpperArm_Bone);
AddChildren(lUpperArm_Bone, lLowerArm_Bone);
AddChildren(Spine0_Bone, rUpperleg_Bone);
AddChildren(rUpperleg_Bone, rLowerleg_Bone);
AddChildren(Spine0_Bone, lUpperleg_Bone);
AddChildren(lUpperleg_Bone, lLowerleg_Bone);

// Store all bones in an array for easier access
SetLength(Bone, 11);
Bone[ 0] := Spine0_Bone;
Bone[ 1] := Spine1_Bone;
Bone[ 2] := Head_Bone;
Bone[ 3] := rUpperArm_Bone;
Bone[ 4] := rLowerArm_Bone;
Bone[ 5] := lUpperArm_Bone;
Bone[ 6] := lLowerArm_Bone;
Bone[ 7] := rUpperleg_Bone;
Bone[ 8] := rLowerleg_Bone;
Bone[ 9] := lUpperleg_Bone;
Bone[10] := lLowerleg_Bone;

// Physics

//create a ragdoll container
RagDoll := NewtonCreateRagDoll(NewtonWorld);
// begin adding bones to the rad doll
NewtonRagDollBegin(RagDoll);
// set the force function
NewtonRagDollSetForceAndTorqueCallback(RagDoll, ApplyForceAndTorqueEvent);
// Build ragdoll
BuildRagDoll(RagDoll, nil, pVelocity, Spine0_Bone);
// finalize the rag doll
NewtonRagDollEnd(RagDoll);
if pHangByHead then
 begin
 // hang ragdoll by the head
 // the head bone is bone #2
 HeadBone := NewtonRagDollFindBone(RagDoll, 2);
 HeadBody := NewtonRagDollBoneGetBody(HeadBone);
 // get the pivot point
 NewtonBodyGetMatrix(HeadBody, @Mat[0,0]);
 Pivot := V3(Mat[3,0], Mat[3,1], Mat[3,2]);
 NewtonConstraintCreateBall(NewtonWorld, @Pivot[0], HeadBody, nil);
 end;
end;

// =============================================================================
//  TRagDoll.BuildRagDoll
// =============================================================================
//  Build the bones of the ragdoll (recursive calls all childrens)
// =============================================================================
procedure TRagDoll.BuildRagDoll(pRagDoll : PNewtonRagDoll; pParentDollBone : PNewtonRagDollBone; pVelocity : TVector3f; var pBone : TRagDollBone);
var
 Body       : PNewtonBody;
 DollBone   : PNewtonRagDollBone;
 Size       : TVector3f;
 Matrix     : TMatrix4f;
 ConeDir    : TVector3f;
 LateralDir : TVector3f;
 i          : Integer;
begin
Size   := pBone.Size;
Matrix := pBone.Matrix;
// create a rag doll bone at the origin of this bone
DollBone := NewtonRagDollAddBone(pRagDoll, pParentDollBone, nil, 10, @Matrix[0], @Size[0]);
NewtonRagDollBoneSetID(DollBone, pBone.ID);
// set the rad doll bone limits
// the x axis is usually the box main axis
ConeDir := V3(1, 0, 0);
// the z axis is usually the box plane
LateralDir := V3(0, 0, 1);
// the axis limits are specifiied in global space, rotate by the joint matrix
Matrix_RotateVect(pBone.Matrix, ConeDir);
Matrix_RotateVect(pBone.Matrix, LateralDir);
// set the limis
NewtonRagDollBoneSetLimits(DollBone, @ConeDir[0], -PI*0.25, PI*0.25, -PI*0.5, @LateralDir[0], 0, 0);
// get the bone rigid body and freeze it
Body := NewtonRagDollBoneGetBody(DollBone);
//NewtonWorldFreezeBody(NewtonWorld, Body);
//uncomment this two lines to make the rag doll active when start
//NewtonWorldUnfreezeBody (NewtonWorld, body);
// set this secuind patameter to 1 to make the ragdoll alwas active (PC and NPC)
//NewtonBodySetAutoFreeze(Body, 0);
// recusivelly add old bones to the rag doll
pBone.Body := Body;
if Length(pBone.Children) > 0 then
 for i := 0 to High(pBone.Children) do
  BuildRagDoll(pRagDoll, DollBone, pVelocity, pBone.Children[i]);
end;

// =============================================================================
//  TRagDoll.Destroy
// =============================================================================
destructor TRagDoll.Destroy;
var
 i : Integer;
begin
for i := 0 to High(Bone) do
 Bone[i].Free;
end;

// =============================================================================
//  TRagDoll.Render
// =============================================================================
procedure TRagDoll.Render;
var
 i : Integer;
begin
for i := 0 to High(Bone) do
 Bone[i].Render;
end;

// =============================================================================
//  GenericContactBegin
// =============================================================================
function GenericContactBegin(const Material : PNewtonMaterial; const Body0 : PNewtonBody; const Body1 : PNewtonBody) : Integer; cdecl;
begin
// Find out if this is the players ball (as it's the only object that has userdata assigned)
UserData := NewtonBodyGetUserData(Body0);
if UserData = nil then
 UserData := NewtonBodyGetUserData(Body1);
Result := 1;
end;

// =============================================================================
//  GenericContactProcess
// =============================================================================
function GenericContactProcess(const Material : PNewtonMaterial; const Contact : PNewtonContact) : Integer; cdecl;
begin
// If one of the colliders was the player's ball, then let it jump again
if UserData <> nil then
 Boolean(UserData^) := True;
Result := 1;
end;

// =============================================================================
//  TGLForm.BuildFont
// =============================================================================
procedure TGLForm.BuildFont(pFontName : String);
var
 Font : HFONT;
begin
FontBase := glGenLists(96);
Font     := CreateFont(12, 0, 0, 0, FW_MEDIUM, 0, 0, 0, ANSI_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, FF_DONTCARE or DEFAULT_PITCH, PChar(pFontName));
SelectObject(DC, Font);
wglUseFontBitmaps(DC, 0, 256, FontBase);
DeleteObject(Font)
end;

// =============================================================================
//  TGLForm.PrintText
// =============================================================================
procedure TGLForm.PrintText(pX,pY : Integer; const pText : String);
begin
glPushAttrib(GL_LIST_BIT);
 glRasterPos2i(pX, pY);
 glListBase(FontBase);
 glCallLists(Length(pText), GL_UNSIGNED_BYTE, PChar(pText));
glPopAttrib;
end;

// =============================================================================
//  TGLForm.ShowText
// =============================================================================
procedure TGLForm.ShowText;
begin
// Show newton logo
glColor3f(1,1,1);
glDisable(GL_LIGHTING);
glDisable(GL_DEPTH_TEST);
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
glOrtho(0,640,480,0, -1,1);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
glEnable(GL_BLEND);
glEnable(GL_TEXTURE_2D);
glBlendFunc(GL_ONE, GL_ONE);
glBindTexture(GL_TEXTURE_2D, NStripTexID);
glBegin(GL_QUADS);
 glTexCoord2f(0,1); glVertex3f(490, 441, 0);
 glTexCoord2f(1,1); glVertex3f(640, 441, 0);
 glTexCoord2f(1,0); glVertex3f(640, 480, 0);
 glTexCoord2f(0,0); glVertex3f(490, 480, 0);
glEnd;
glDisable(GL_BLEND);
glDisable(GL_TEXTURE_2D);
// Show text
PrintText(5,15, FloatToStr(FPS)+' fps');
PrintText(5,30, '"R" - Reset ball to next spawn point');
PrintText(5,43, '"Return" - Jump');
PrintText(5,56, '"Cursor keys" - Move');
PrintText(5,69, '"Left mouse + drag" - Rotate ball direction');
PrintText(5,84, '"Mousewheel" - Zoom');
PrintText(5,97, '"Escape" - Quit');
glEnable(GL_DEPTH_TEST);
glEnable(GL_TEXTURE_2D);
end;

// =============================================================================
//  TGLForm.FormCreate
// =============================================================================
procedure TGLForm.FormCreate(Sender: TObject);
begin
InitOpenGL;
DC := GetDC(Handle);
RC := CreateRenderingContext(DC, [opDoubleBuffered], 32, 24, 0, 0, 0, 0);
ActivateRenderingContext(DC, RC);
glEnable(GL_DEPTH_TEST);
glEnable(GL_NORMALIZE);
glEnable(GL_COLOR_MATERIAL);
glClearColor(0,0,0,0);
glDepthFunc(GL_LEQUAL);
LoadTexture('data\textures\loading.jpg', LoadTexID);
QueryPerformanceFrequency(QPCFreq);
InitTimer.Enabled := True;
end;

// =============================================================================
//  TGLForm.InitTimerTimer
// =============================================================================
procedure TGLForm.InitTimerTimer(Sender: TObject);
var
 Key : Char;
begin
InitTimer.Enabled := False;

// Let the user select a 3DS file (if not set by a parameter)
if ParamCount = 0 then
 begin
 Open3DSDialog.InitialDir := ExtractFilePath(Application.ExeName)+'data\scenes';
 if not Open3DSDialog.Execute then
  Application.Terminate
 else
  File3DS := Open3DSDialog.FileName;
 end
else
 File3DS := ParamStr(1);

if not FileExists(File3DS) then
 Application.Terminate;

ChDir(ExtractFilePath(Application.ExeName));
// Show a slpash screen (with OpenGL) while loading data
glMatrixMode(GL_PROJECTION);
glLoadIdentity;
glOrtho(0, 1, 1, 0, -1,1);
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
glEnable(GL_TEXTURE_2D);
glBindTexture(GL_TEXTURE_2D, LoadTexID);
glBegin(GL_QUADS);
 glTexCoord2f(0,1); glVertex3f(0, 0, 0);
 glTexCoord2f(1,1); glVertex3f(1, 0, 0);
 glTexCoord2f(1,0); glVertex3f(1, 1, 0);
 glTexCoord2f(0,0); glVertex3f(0, 1, 0);
glEnd;
glDisable(GL_BLEND);
SwapBuffers(DC);
BuildFont('MS Sans Serif');
Application.OnIdle := ApplicationEventsIdle;
StartTick := GetTickCount;
Init := True;
NewtonWorld := NewtonCreate(nil, nil);

// Set some default values
cameraDir      := V3(0, 0, -1);
cameraEyepoint := V3(0, 15, 0);
Zoom           := 4;
yawAngle       := 90 * 3.1416 / 180;
rollAngle      := 0;

// Generate materials
// Default (3DS scene, although a collision between this material and something of the same material will never happen)
MatWorld  := NewtonMaterialGetDefaultGroupID(NewtonWorld);
NewtonMaterialSetDefaultSoftness(NewtonWorld, MatWorld, MatWorld, 0.05);
NewtonMaterialSetDefaultElasticity(NewtonWorld, MatWorld, MatWorld, 0.4);
NewtonMaterialSetDefaultCollidable(NewtonWorld, MatWorld, MatWorld, 1);
NewtonMaterialSetDefaultFriction(NewtonWorld, MatWorld, MatWorld, 1, 0.5);
NewtonMaterialSetCollisionCallback(NewtonWorld, MatWorld, MatWorld, nil, @GenericContactBegin, @GenericContactProcess, nil);
// Player ball
MatSphere := NewtonMaterialCreateGroupID(NewtonWorld);
NewtonMaterialSetDefaultElasticity(NewtonWorld, MatWorld, MatSphere, 0.3);
NewtonMaterialSetDefaultFriction(NewtonWorld, MatWorld, MatSphere, 1.1, 0.7);
NewtonMaterialSetCollisionCallback(NewtonWorld, MatWorld, MatSphere, nil, @GenericContactBegin, @GenericContactProcess, nil);

// Load 3DS scene
Scene3DS := TAll3DSMesh.Create(self);
Scene3DS.TexturePath := 'data\textures\';
Scene3DS.LoadFromFile(File3DS);
Create3DSCollisionTree(Scene3DS);

// Create player ball
Collider := NewtonCreateSphere(NewtonWorld, 0.5, nil);
BallBody := NewtonCreateBody(NewtonWorld, Collider);
Matrix_SetIdentity(BallMatrix);
Matrix_SetTransform(BallMatrix, V4(-5,10,2));
NewtonBodySetMatrix(BallBody, @BallMatrix[0,0]);
NewtonReleaseCollision(NewtonWorld, Collider);
NewtonBodySetForceAndTorqueCallback(BallBody, BallCharacterApplyForceAndTorque);
NewtonBodySetMaterialGroupID(BallBody, MatSphere);
// Give it a huge mass, so it makes fun to throw them against other crates ;)
NewtonBodySetMassMatrix(BallBody, 10, 10, 10, 10);
NewtonWorldUnfreezeBody(NewtonWorld, BallBody);
NewtonBodySetAutoFreeze(BallBody, 0);
// Set userdata to a bool indicating weather the player can jump or not
NewtonBodySetUserData(BallBody, @CanJump);

SetLength(RagDoll, 1);
RagDoll[0]  := TRagDoll.Create(V3(-56,27,36), V3(0,0,0), False);

// Load some textures
LoadTexture('data\textures\sphere.jpg', SphereTexID);
LoadTexture('data\textures\skysphere.jpg', SkyTexID);
LoadTexture('data\textures\newtonstrip.jpg', NStripTexID);
glDisable(GL_TEXTURE_2D);

// Generate quadric for the ball and the syksphere
Q := gluNewQuadric;
// Generate a display list for the player ball (faster rendering)
SphereListID := glGenLists(1);
glNewList(SphereListID, GL_COMPILE);
 glColor3f(1,1,1);
 glBindTexture(GL_TEXTURE_2D, SphereTexID);
 glEnable(GL_TEXTURE_2D);
 glMaterialf(GL_FRONT, GL_SHININESS, 50.0);
 gluQuadricTexture(Q, True);
 gluSphere(Q, 0.5, 24, 24);
 glMaterialf(GL_FRONT, GL_SHININESS, 1.0);
 glDisable(GL_TEXTURE_2D);
glEndList;
// Generate display list for the skydome (faster rendering)
SkySphereListID := glGenLists(1);
glNewList(SkySphereListID, GL_COMPILE);
 glColor3f(1,1,1);
 glCullFace(GL_FRONT);
 glDisable(GL_LIGHTING);
 glEnable(GL_TEXTURE_2D);
 glBindTexture(GL_TEXTURE_2D, SkyTexID);
 glDepthMask(False);
 glRotatef(-90, 1,0,0);
 gluSphere(Q, 20, 16, 16);
 glDepthMask(True);
 glDisable(GL_TEXTURE_2D);
 glCullFace(GL_BACK);
glEndList;
// Delete quadric again, as it's stored in the display lists
gluDeleteQuadric(Q);
// Rest ball to first spawn point
CurrStartPoint := 0;
Key := 'r';
FormKeyPress(nil, Key);
end;


// =============================================================================
//  TGLForm.FormDestroy
// =============================================================================
procedure TGLForm.FormDestroy(Sender: TObject);
var
I : Integer;
begin
if Init then
 begin
 DeactivateRenderingContext;
 wglDeleteContext(RC);
 ReleaseDC(Handle, DC);
 if Length(RagDoll) > 0 then
 for i := 0 to High(RagDoll) do
  RagDoll[i].Free;
 NewtonDestroy(NewtonWorld);
 end;
end;

// =============================================================================
//  TGLForm.Render
// =============================================================================
procedure TGLForm.Render;
var
I : Integer;
begin
// Render the skydome 
glPushMatrix;
 glTranslatef(cameraEyepoint[0], cameraEyepoint[1], cameraEyepoint[2]);
 glCallList(SkySphereListID);
glPopMatrix;
// Set light
glEnable(GL_LIGHTING);
glEnable(GL_LIGHT0);
glLightfv(GL_LIGHT0, GL_POSITION, @LightPos[0]);

// Render scene
glCullFace(GL_FRONT);
glEnable(GL_TEXTURE_2D);
Scene3DS.Render;
glCullFace(GL_BACK);
// Render ball
NewtonBodyGetMatrix(BallBody, @BallMatrix[0,0]);
glMultMatrixf(@BallMatrix);
glCallList(SphereListID);

glLoadIdentity;
glTranslatef(cameraEyepoint[0], cameraEyepoint[1], cameraEyepoint[2]);

// Render all ragdolls
if Length(RagDoll) > 0 then
 for i := 0 to High(RagDoll) do
  RagDoll[i].Render;

end;

// =============================================================================
//  TGLForm.ApplicationEventsIdle
// =============================================================================
procedure TGLForm.ApplicationEventsIdle(Sender: TObject; var Done: Boolean);
var
 QPCStart : Int64;
 QPCEnd   : Int64;
 M        : TMatrix4f;
begin
if not Init then
 exit;

QueryPerformanceCounter(QPCStart);

// Get position of the ball and update the camera's eyepoint
NewtonBodyGetMatrix(BallBody, @M);
cameraEyepoint := V3(M[3,0]-cameraDir[0]*Zoom, M[3,1]-cameraDir[1]*Zoom, M[3,2]-cameraDir[2]*Zoom);

glMatrixMode(GL_PROJECTION);
glLoadIdentity;
glViewPort(0, 0, ClientWidth, ClientHeight);
gluPerspective(FOV, ClientWidth/ClientHeight, zNear, zFar);
glMatrixMode(GL_MODELVIEW);
glLoadIdentity;
glClear(GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT);
// Set camera to follow the ball
gluLookAt(cameraEyepoint[0],cameraEyepoint[1],cameraEyepoint[2],
          cameraEyepoint[0]+cameraDir[0],cameraEyepoint[1]+cameraDir[1],cameraEyepoint[2]+cameraDir[2], 0,1,0);
// Render scene
Render;

glCullFace(GL_FRONT);
ShowText;

SwapBuffers(DC);
Done := False;

QueryPerformanceCounter(QPCEnd);
// Update newton world
UserData := nil;
NewtonUpdate(NewtonWorld, ((QPCEnd-QPCStart)/QPCFreq*1000)*0.0015);

inc(Frames);
if GetTickCount - StartTick >= 500 then
 begin
 FPS       := Frames/(GetTickCount-StartTick)*1000;
 Frames    := 0;
 StartTick := GetTickCount
 end;
end;


// =============================================================================
//  TForm1.FormKeyPress
// =============================================================================
procedure TGLForm.FormKeyPress(Sender: TObject; var Key: Char);
const
 VZero : TVector3f = (0,0,0);
var
 TmpV  : TVector3f;
 m,n   : Integer;
begin
case Key of
 #27 : Close;
 #13 : if CanJump then
        begin
        NewtonBodyGetVelocity(BallBody, @TmpV[0]);
        TmpV := V3(TmpV[0], TmpV[1]+25, TmpV[2]);
        NewtonBodySetVelocity(BallBody, @TmpV[0]);
        CanJump := False;
        end;
 'r' : begin
       // Stop all ball motions
       NewtonBodySetVelocity(BallBody, @VZero);
       NewtonBodySetOmega(BallBody, @VZero);
       NewtonBodySetTorque(BallBody, @VZero);
       // Reset ball to next start point in list
       for m := 0 to Scene3DS.NumMeshes-1 do
        if Scene3DS.Mesh[m].Name = 'start'+IntToStr(CurrStartPoint) then
         begin
         Matrix_SetIdentity(BallMatrix);
         with Scene3DS.Mesh[m] do
          Matrix_SetTransform(BallMatrix, V4(Minimum.x+(Maximum.x-Minimum.x)/2,
                                             Minimum.y+(Maximum.y-Minimum.y)/2,
                                             Minimum.z+(Maximum.z-Minimum.z)/2));
         NewtonBodySetMatrix(BallBody, @BallMatrix[0,0]);
         // Set next spawn point
         for n := 0 to Scene3DS.NumMeshes-1 do
          if Scene3DS.Mesh[n].Name = 'start'+IntToStr(CurrStartPoint+1) then
           begin
           Inc(CurrStartPoint);
           exit;
           end;
         // No next start point found, set to zero
         CurrStartPoint := 0;
         exit;
         end;
       // If no start points in scene, reset to zero position
       Matrix_SetIdentity(BallMatrix);
       Matrix_SetTransform(BallMatrix, V4(0,10,0));
       NewtonBodySetMatrix(BallBody, @BallMatrix[0,0]);
       end;
end;
end;

// =============================================================================
//  TGLForm.FormKeyDown
// =============================================================================
procedure TGLForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
case Key of
 vk_Up    : cameraTorqueVector[2] := -1;
 vk_Down  : cameraTorqueVector[2] :=  1;
 vk_Left  : cameraTorqueVector[0] := -1;
 vk_Right : cameraTorqueVector[0] :=  1;
end;
end;

// =============================================================================
//  TGLForm.FormKeyUp
// =============================================================================
procedure TGLForm.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
cameraTorqueVector := V3(0.0, 0.0, 0.0);
end;

// =============================================================================
//  TGLForm.FormMouseMove
// =============================================================================
procedure TGLForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
var
 Mouse1 : TPoint;
begin
// read the mouse position and set the camera direction
GetCursorPos(Mouse1);
if ssLeft in Shift then
 begin
 if Mouse1.x > Mouse0.x then
  begin
 	yawAngle := yawAngle + 5 * PIOver180;
  if yawAngle > (360 * PIOver180) then
   yawAngle := yawAngle - (360 * PIOver180);
  end;
 if Mouse1.x < Mouse0.x  then
  begin
  yawAngle := yawAngle - 5 * PIOver180;
	if yawAngle < 0 then
 	 yawAngle := yawAngle + (360 * PIOver180);
  end;
 if Mouse1.y > Mouse0.y then
  begin
  rollAngle := rollAngle + 2 * PIOver180;
  if rollAngle > (80 * PIOver180) then
   rollAngle := 80 * PIOver180;
  end;
 if Mouse1.y < Mouse0.y then
  begin
  rollAngle := rollAngle - 2 * PIOver180;
  if rollAngle < -(80 * PIOver180) then
   rollAngle := -80 * PIOver180;
  end;
 cameraDirMat := Matrix_Multiply(Matrix_MakeRollMatrix(rollAngle), Matrix_MakeYawMatrix(yawAngle));
 cameraDir    := V3(cameraDirMat[0,0], cameraDirMat[0,1], cameraDirMat[0,2]);
 end;
Mouse0 := Mouse1;
end;

// =============================================================================
//  TGLForm.FormMouseWheel
// =============================================================================
procedure TGLForm.FormMouseWheel(Sender: TObject; Shift: TShiftState; WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
Zoom := Zoom - 0.01 * WheelDelta;
end;

end.

