(*******************************************************************************
* Desc. Translation of the Newton headers to Delphi language
* Delphi 5.0 and up are supported(i hope)
*
* Author: S.Spasov (Sury 2003)
* Sofia Bulgaria
* spas_gen@mail.bg
*
* Feel free to use this unit in whatever project you want !
* If you make modifications or find any bugs please mail me.
*
* Note : Commented out functions was commented out in the
* original Newton header too , thus no translation is provided.
*
* History :
* Conversion started at 13.08.2004 by Sury
* Conversion completed at 13.08.2004 by Sury
* Non-Delphi compiler support added on 17.08.2008 by Dominique Louis
*******************************************************************************)

unit newtonimport;

{$I delphinewton.inc}

interface

uses
{$IFDEF __GPC__}
  system,
  gpc,
{$ENDIF}

{$IFDEF UNIX}
{$IFDEF FPC}
{$IFDEF Ver1_0}
  linux,
{$ELSE}
  pthreads,
  baseunix,
  unix,
{$ENDIF}
  x,
  xlib,
{$ELSE}
  Types,
  Libc,
  Xlib,
{$ENDIF}
{$ENDIF}

{$IFDEF __MACH__}
  GPCMacOSAll,
{$ENDIF}
  Classes;

const
{$IFDEF WIN32}
  NewtonDLL = 'Newton.dll';
{$ENDIF}

{$IFDEF UNIX}
{$IFDEF DARWIN} // MacOS X
  NewtonDLL = 'libnewton.dylib';
{$ELSE}
  NewtonDLL = 'libnewton.so';
{$ENDIF}
{$ENDIF}

{$IFDEF MACOS}
  NewtonDLL = 'libnewton';
{$ENDIF}

(*Comment this line if you get weird errors*)
{$DEFINE NICE_CODE_PARAMS}

type
{I did this to speed up the translation process and avoid bugs}
{if you don't like me screw up the Delphi syntax with those
(C++ types just do a simple find and replace =)

{Pascal to C++}
  {simple types}
  Bool = Boolean;
  Float = Single;
  Long_double = Extended;

  Int = Integer;
  __int8 = ShortInt;
  __int16 = SmallInt;
  __int32 = LongInt;
  __int64 = Int64;
  NChar = ShortInt;
  Unsigned_char = Byte;
  Short = SmallInt;
  Unsigned_short = Word;
  Long = LongInt;
  Unsigned_long = LongWord;
  Unsigned_int = Cardinal;
  size_t = Cardinal;

  {Pointer types}
  Pvoid = Pointer; //void pointer
  PBool = ^Bool;
  PFloat = ^Float;
  PLong_double = ^Long_double;

  PInt = ^Int;
  P__int8 = ^__int8;
  P__int16 = ^__int16;
  P__int32 = ^__int32;
  P__int64 = ^__int64;
  P2Char = ^NChar;
  PUnsigned_char = ^Unsigned_char;
  PShort = ^Short;
  PUnsigned_short = ^Unsigned_short;
  PLong = ^Long;
  PUnsigned_long = ^Unsigned_long;
  PUnsigned_int = ^Unsigned_int;
  Psize_t = ^size_t;

{end Pascal to C++}

  {well this might look stupid
  but i did it in order to make
  code complete and code parameters hint window
  to show the actual type for ex. PNewtonWorld
  instead of just "Pointer" , thus making programming a lot easier}
{$IFDEF NICE_CODE_PARAMS}
  PNewtonBody = ^Pointer;
  PNewtonWorld = ^Pointer;
  PNewtonJoint = ^Pointer;
  PNewtonContact = ^Pointer;
  PNewtonMaterial = ^Pointer;
  PNewtonCollision = ^Pointer;

  PNewtonRagDoll = ^Pointer;
  PNewtonRagDollBone = ^Pointer;
{$ELSE}

  PNewtonBody = Pointer;
  PNewtonWorld = Pointer;
  PNewtonJoint = Pointer;
  PNewtonContact = Pointer;
  PNewtonMaterial = Pointer;
  PNewtonCollision = Pointer;

  PNewtonRagDoll = Pointer;
  PNewtonRagDollBone = Pointer;
{$ENDIF}



(*
 typedef struct NewtonUserMeshCollisionCollideDescTag
{
  float m_boxP0[4];
  float m_boxP1[4];
  void* m_userData;
  int	  m_faceCount;
  float* m_vertex;
  int m_vertexStrideInBytes;
  int* m_userAttribute;
  int* m_faceIndexCount;
  int* m_faceVertexIndex;
  NewtonBody* m_objBody;
  NewtonBody* m_polySoupBody;
  NewtonUserMeshCollisionCollideDesc;
}
*)


  NewtonUserMeshCollisionCollideDesc = record

    m_boxP0 : array[ 0..3 ] of float;
    m_boxP1 : array[ 0..3 ] of float;
    m_userData : Pointer;
    m_faceCount : int;
    m_vertex : ^float;
    m_vertexStrideInBytes : int;
    m_userAttribute : ^int;
    m_faceIndexCount : ^int;
    m_faceVertexIndex : ^int;
    m_objBody : PNewtonBody;
    m_polySoupBody : PNewtonBody;
  end;
  PNewtonUserMeshCollisionCollideDesc = ^NewtonUserMeshCollisionCollideDesc;

(*
typedef struct NewtonUserMeshCollisionRayHitDescTag
 {
  float m_p0[4];
  float m_p1[4];
  float m_pointOut[4];
  float m_normalOut[4];
  int m_userId;
  float m_thickness;
  void*  m_userData;
  NewtonUserMeshCollisionRayHitDesc;
}
*)

  NewtonUserMeshCollisionRayHitDesc = record

    m_p0 : array[ 0..3 ] of float;
    m_p1 : array[ 0..3 ] of float;
    m_pointOut : array[ 0..3 ] of float;
    m_normalOut : array[ 0..3 ] of float;
    m_userId : int;
    m_thickness : float;
    m_userData : Pointer;
  end;
  PNewtonUserMeshCollisionRayHitDesc = ^NewtonUserMeshCollisionRayHitDesc;
(*
typedef struct NewtonHingeSliderUpdateDescTag
{
  float m_accel;
  float m_minFriction;
  float m_maxFriction;
  float m_timestep;
} NewtonHingeSliderUpdateDesc;
*)

  NewtonHingeSliderUpdateDesc = record
    m_accel : float;
    m_minFriction : float;
    m_maxFriction : float;
    m_timestep : float;
  end;
  PNewtonHingeSliderUpdateDesc = ^NewtonHingeSliderUpdateDesc;
(*
// Newton callback functions
 typedef void* (_cdecl *NewtonAllocMemory) (int sizeInBytes);
 typedef void (_cdecl *NewtonFreeMemory) (void *ptr, int sizeInBytes);

 typedef void (_cdecl *NewtonSerialize) (void* serializeHandle, const void* buffer, size_t size);
 typedef void (_cdecl *NewtonDeserialize) (void* serializeHandle, void* buffer, size_t size);


 typedef void (_cdecl *NewtonUserMeshCollisionCollideCallback) (NewtonUserMeshCollisionCollideDesc* collideDescData);
 typedef int  (_cdecl *NewtonUserMeshCollisionRayHitCallback) (NewtonUserMeshCollisionRayHitDesc* lineDescData);
 typedef void (_cdecl *NewtonUserMeshCollisionDestroyCallback) (void* descData);
 typedef void (_cdecl *NewtonTreeCollisionCallback) (const NewtonBody* bodyWithTreeCollision, const NewtonBody* body,
  const float* vertex, int vertexstrideInBytes,
  int indexCount, const int* indexArray);

 typedef void (_cdecl *NewtonBodyDestructor) (const NewtonBody* body);
 typedef void (_cdecl *NewtonApplyForceAndTorque) (const NewtonBody* body);
 typedef void (_cdecl *NewtonBodyActivationState) (const NewtonBody* body, unsigned state);
 typedef void (_cdecl *NewtonSetTransform) (const NewtonBody* body, const float* matrix);
 typedef void (_cdecl *NewtonSetRagDollTransform) (const NewtonRagDollBone* bone);
 typedef void (_cdecl *NewtonGetBuoyancyPlane) (void *context, const float* globalSpaceMatrix, float* globalSpacePlane);


 typedef void (_cdecl *NewtonBodyLeaveWorld) (const NewtonBody* body);

 typedef int  (_cdecl *NewtonContactBegin) (const NewtonMaterial* material, const NewtonBody* body0, const NewtonBody* body1);
 typedef int  (_cdecl *NewtonContactProcess) (const NewtonMaterial* material, const NewtonContact* contact);
 typedef void (_cdecl *NewtonContactEnd) (const NewtonMaterial* material);


 typedef void (_cdecl *NewtonBodyIterator) (const NewtonBody* body);
 typedef void (_cdecl *NewtonCollisionIterator) (const NewtonBody* body, int vertexCount, const float* FaceArray, int faceId);


 typedef void (_cdecl *NewtonBallCallBack) (const NewtonJoint* ball);
 typedef void (_cdecl *NewtonUpVectorCallBack) (const NewtonJoint* upVector);
 typedef unsigned (_cdecl *NewtonHingeCallBack) (const NewtonJoint* hinge, NewtonHingeSliderUpdateDesc* desc);
 typedef unsigned (_cdecl *NewtonSliderCallBack) (const NewtonJoint* slider, NewtonHingeSliderUpdateDesc* desc);
 typedef unsigned (_cdecl *NewtonUniversalCallBack) (const NewtonJoint* universal, NewtonHingeSliderUpdateDesc* desc);
 typedef unsigned (_cdecl *NewtonCorkscrewCallBack) (const NewtonJoint* corkscrew, NewtonHingeSliderUpdateDesc* desc);

 typedef void (_cdecl *NewtonConstraintDestructor) (const NewtonJoint* me);
*)


(*******************************************************************************
*                           NEWTON CALLBACK FUNCTIONS
*
********************************************************************************)

  NewtonAllocMemory =
    procedure( sizeInBytes : int ); cdecl;
  PNewtonAllocMemory = ^NewtonAllocMemory;

  NewtonFreeMemory =
    procedure( ptr : Pointer; sizeInBytes : int ); cdecl;
  PNewtonFreeMemory = ^NewtonFreeMemory;

  NewtonSerialize =
    procedure( serializeHandle : Pointer; const buffer : Pointer; size : size_t ); cdecl;
  PNewtonSerialize = ^NewtonSerialize;

  NewtonDeserialize =
    procedure( serializeHandle : Pointer; buffer : Pointer; size : size_t ); cdecl;
  PNewtonDeserialize = ^NewtonDeserialize;

  NewtonUserMeshCollisionCollideCallback =
    procedure( NewtonUserMeshCollisionCollideDesc : PNewtonUserMeshCollisionCollideDesc ); cdecl;
  PNewtonUserMeshCollisionCollideCallback = ^NewtonUserMeshCollisionCollideCallback;


  NewtonUserMeshCollisionRayHitCallback =
    function( NewtonUserMeshCollisionRayHitDesc : PNewtonUserMeshCollisionRayHitDesc ) : int; cdecl;
  PNewtonUserMeshCollisionRayHitCallback = ^NewtonUserMeshCollisionRayHitCallback;

  NewtonUserMeshCollisionDestroyCallback =
    procedure( descData : Pointer ); cdecl;
  PNewtonUserMeshCollisionDestroyCallback = ^NewtonUserMeshCollisionDestroyCallback;

  NewtonTreeCollisionCallback =
    procedure( const bodyWithTreeCollision : PNewtonBody; const body : PNewtonBody;
    const vertex : PFloat; vertexstrideInBytes : int;
    indexCount : int; const indexArray : PInt ); cdecl;

  PNewtonTreeCollisionCallback = ^NewtonTreeCollisionCallback;

  NewtonBodyDestructor =
    procedure( const body : PNewtonBody ); cdecl;
  PNewtonBodyDestructor = ^NewtonBodyDestructor;

  NewtonApplyForceAndTorque =
    procedure( const body : PNewtonBody ); cdecl;
  PNewtonApplyForceAndTorque = ^NewtonApplyForceAndTorque;

  NewtonBodyActivationState =
    procedure( const body : PNewtonBody; state : unsigned_int ); cdecl;
  PNewtonBodyActivationState = ^NewtonBodyActivationState;

  NewtonSetTransform =
    procedure( const body : PNewtonBody; const matrix : PFloat ); cdecl;
  PNewtonSetTransform = ^NewtonSetTransform;


  NewtonSetRagDollTransform =
    procedure( const bone : PNewtonRagDollBone ); cdecl;
  PNewtonSetRagDollTransform = ^NewtonSetRagDollTransform;

  NewtonGetBuoyancyPlane =
    procedure( context : Pointer; const globalSpaceMatrix : Float; globalSpacePlane : PFloat ); cdecl;
  PNewtonGetBuoyancyPlane = ^NewtonGetBuoyancyPlane;


  NewtonBodyLeaveWorld =
    procedure( const body : PNewtonBody ); cdecl;
  PNewtonBodyLeaveWorld = ^NewtonBodyLeaveWorld;

  NewtonContactBegin =
    function( const material : PNewtonMaterial; const body0 : PNewtonBody; const body1 : PNewtonBody ) : int; cdecl;
  PNewtonContactBegin = ^NewtonContactBegin;


  NewtonContactProcess =
    function( const material : PNewtonMaterial; const contact : PNewtonContact ) : int; cdecl;
  PNewtonContactProcess = ^NewtonContactProcess;

  NewtonContactEnd =
    procedure( const material : PNewtonMaterial ); cdecl;
  PNewtonContactEnd = ^NewtonContactEnd;


  NewtonBodyIterator =
    procedure( const body : PNewtonBody ); cdecl;
  PNewtonBodyIterator = ^NewtonBodyIterator;

  NewtonCollisionIterator =
    procedure( const body : PNewtonBody; vertexCount : int; const FaceArray : PFloat; faceId : int );
  PNewtonCollisionIterator = ^NewtonCollisionIterator;

  NewtonBallCallBack =
    procedure( const ball : PNewtonJoint ); cdecl;
  PNewtonBallCallBack = ^NewtonBallCallBack;


  NewtonUpVectorCallBack =
    procedure( const upVector : PNewtonJoint ); cdecl;
  PNewtonUpVectorCallBack = ^NewtonUpVectorCallBack;


  NewtonHingeCallBack =
    function( const hinge : PNewtonJoint; desc : PNewtonHingeSliderUpdateDesc ) : Unsigned_int; cdecl;
  PNewtonHingeCallBack = ^NewtonHingeCallBack;


  NewtonSliderCallBack =
    function( const slider : PNewtonJoint; desc : PNewtonHingeSliderUpdateDesc ) : Unsigned_int; cdecl;
  PNewtonSliderCallBack = ^NewtonSliderCallBack;

  NewtonUniversalCallBack =
    function( const universal : PNewtonJoint; desc : PNewtonHingeSliderUpdateDesc ) : Unsigned_int; cdecl;
  PNewtonUniversalCallBack = ^NewtonUniversalCallBack;

  NewtonCorkscrewCallBack =
    function( const corkscrew : PNewtonJoint; desc : PNewtonHingeSliderUpdateDesc ) : Unsigned_int; cdecl;
  PNewtonCorkscrewCallBack = ^NewtonCorkscrewCallBack;

  NewtonConstraintDestructor =
    procedure( const me : PNewtonJoint ); cdecl;
  PNewtonConstraintDestructor = ^NewtonConstraintDestructor;


 // **********************************************************************************************
 //
 // world control functions
 //
 // **********************************************************************************************
function NewtonCreate( malloc : PNewtonAllocMemory; mfree : PNewtonFreeMemory ) : PNewtonWorld; cdecl; external{$IFDEF __GPC__}name 'NewtonCreate'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonDestroy( const newtonWorld : PNewtonWorld ); cdecl; external{$IFDEF __GPC__}name 'NewtonDestroy'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonDestroyAllBodies( const newtonWorld : PNewtonWorld ); cdecl; external{$IFDEF __GPC__}name 'NewtonDestroyAllBodies'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonUpdate( const newtonWorld : PNewtonWorld; timestep : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonUpdate'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonSetMinimumFrameRate( const newtonWorld : PNewtonWorld; frameRate : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonSetMinimumFrameRate'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonSetBodyLeaveWorldEvent( const newtonWorld : PNewtonWorld; callback : PNewtonBodyLeaveWorld ); cdecl; external{$IFDEF __GPC__}name 'NewtonSetBodyLeaveWorldEvent'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonSetWorldSize( const newtonWorld : PNewtonWorld; const minPoint : Pfloat; const maxPoint : pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonSetWorldSize'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonWorldFreezeBody( const newtonWorld : PNewtonWorld; const body : PNewtonBody ); cdecl; external{$IFDEF __GPC__}name 'NewtonWorldFreezeBody'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonWorldUnfreezeBody( const newtonWorld : PNewtonWorld; const body : PNewtonBody ); cdecl; external{$IFDEF __GPC__}name 'NewtonWorldUnfreezeBody'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonWorldForEachBodyDo( const newtonWorld : PNewtonWorld; callback : PNewtonBodyIterator ); cdecl; external{$IFDEF __GPC__}name 'NewtonWorldForEachBodyDo'{$ELSE}NewtonDLL{$ENDIF __GPC__};

 // **********************************************************************************************
 //
 // Physics Material Section
 //
 // **********************************************************************************************
function NewtonMaterialGetDefaultGroupID( const newtonWorld : PNewtonWorld ) : int; cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialGetDefaultGroupID'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonMaterialCreateGroupID( const newtonWorld : PNewtonWorld ) : int; cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialCreateGroupID'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialDestroyAllGroupID( const newtonWorld : PNewtonWorld ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialDestroyAllGroupID'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetDefaultSoftness( const newtonWorld : PNewtonWorld; id0 : int; id1 : int; value : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetDefaultSoftness'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetDefaultElasticity( const newtonWorld : PNewtonWorld; id0 : int; id1 : int; elasticCoef : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetDefaultElasticity'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetDefaultCollidable( const newtonWorld : PNewtonWorld; id0 : int; id1 : int; state : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetDefaultCollidable'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetDefaultFriction( const newtonWorld : PNewtonWorld; id0 : int; id1 : int; staticFriction : float; kineticFriction : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetDefaultFriction'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetCollisionCallback( const newtonWorld : PNewtonWorld; id0 : int; id1 : int; userData : Pointer; _begin : PNewtonContactBegin; process : PNewtonContactProcess; _end : PNewtonContactEnd ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetCollisionCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};


 // **********************************************************************************************
 //
 // Physics Contact control functions
 //
 // **********************************************************************************************
procedure NewtonMaterialDisableContact( const material : PNewtonMaterial ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialDisableContact'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonMaterialGetContactUserData( const material : PNewtonMaterial ) : Pointer; cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialGetContactUserData'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonMaterialGetContactNormalSpeed( const material : PNewtonMaterial; const contactlHandle : PNewtonContact ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialGetContactNormalSpeed'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonMaterialGetContactTangentSpeed( const material : PNewtonMaterial; const contactlHandle : PNewtonContact; index : int ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialGetContactTangentSpeed'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialGetContactPositionAndNormal( const material : PNewtonMaterial; posit : Pfloat; normal : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialGetContactPositionAndNormal'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialGetContactTangentDirections( const material : PNewtonMaterial; dir0 : Pfloat; dir : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialGetContactTangentDirections'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonMaterialGetBodyCollisionID( const material : PNewtonMaterial; const body : PNewtonBody ) : Unsigned_int; cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialGetBodyCollisionID'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetContactSoftness( const material : PNewtonMaterial; softness : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetContactSoftness'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetContactElasticity( const material : PNewtonMaterial; restitution : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetContactElasticity'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetContactFrictionState( const material : PNewtonMaterial; state : int; index : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetContactFrictionState'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetContactStaticFrictionCoef( const material : PNewtonMaterial; coef : float; index : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetContactStaticFrictionCoef'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetContactKineticFrictionCoef( const material : PNewtonMaterial; coef : float; index : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetContactKineticFrictionCoef'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialSetContactTangentAcceleration( const material : PNewtonMaterial; accel : float; index : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialSetContactTangentAcceleration'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonMaterialContactRotateTangentDirections( const material : PNewtonMaterial; const directionVector : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonMaterialContactRotateTangentDirections'{$ELSE}NewtonDLL{$ENDIF __GPC__};


 // **********************************************************************************************
 //
 // convex collision primitives creation functions
 //
 // **********************************************************************************************
function NewtonCreateSphere( const newtonWorld : PNewtonWorld; radius : float; const offsetMatrix : Pfloat ) : PNewtonCollision; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateSphere'{$ELSE}NewtonDLL{$ENDIF __GPC__};
 //NEWTON_API NewtonCollision* NewtonCreateCylinder (const NewtonWorld* newtonWorld, float radius, float height, const float *offsetMatrix);
function NewtonCreateBox( const newtonWorld : PNewtonWorld; dx : float; dy : float; dz : float; const offsetMatrix : Pfloat ) : PNewtonCollision; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateBox'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonConvexCollisionSetUserID( const convexCollision : PNewtonCollision; id : unsigned_int ); cdecl; external{$IFDEF __GPC__}name 'NewtonConvexCollisionSetUserID'{$ELSE}NewtonDLL{$ENDIF __GPC__};

 // **********************************************************************************************
 //
 // complex collision primitives creation functions
 // note: can only be used with static bodies (bodies with infinite mass)
 //
 // **********************************************************************************************
type
  TcollisionPrimitiveArray = array of PNewtonCollision;

function NewtonCreateCompoundCollision( const newtonWorld : PNewtonWorld; count : int;
  const collisionPrimitiveArray : TcollisionPrimitiveArray ) : PNewtonCollision; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateCompoundCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonCreateUserMeshCollision( const newtonWorld : PNewtonWorld; const minBox : Pfloat;
  const maxBox : Pfloat; userData : Pointer; collideCallback : NewtonUserMeshCollisionCollideCallback;
  rayHitCallback : NewtonUserMeshCollisionRayHitCallback; destroyCallback : NewtonUserMeshCollisionDestroyCallback ) : PNewtonCollision; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateUserMeshCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};


 // **********************************************************************************************
 //
 // CollisionTree Utility funstions
 //
 // **********************************************************************************************
function NewtonCreateTreeCollision( const newtonWorld : PNewtonWorld; userCallback : NewtonTreeCollisionCallback ) : PNewtonCollision; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateTreeCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonTreeCollisionBeginBuild( const treeCollision : PNewtonCollision ); cdecl; external{$IFDEF __GPC__}name 'NewtonTreeCollisionBeginBuild'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonTreeCollisionAddFace( const treeCollision : PNewtonCollision; vertexCount : int; const vertexPtr : Pfloat;
  strideInBytes : int; faceAttribute : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonTreeCollisionAddFace'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonTreeCollisionEndBuild( const treeCollision : PNewtonCollision; optimize : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonTreeCollisionEndBuild'{$ELSE}NewtonDLL{$ENDIF __GPC__};


procedure NewtonTreeCollisionSerialize( const treeCollision : PNewtonCollision; serializeFunction : NewtonSerialize;
  serializeHandle : Pointer ); cdecl; external{$IFDEF __GPC__}name 'NewtonTreeCollisionSerialize'{$ELSE}NewtonDLL{$ENDIF __GPC__};


function NewtonCreateTreeCollisionFromSerialization( const newtonWorld : PNewtonWorld;
  userCallback : NewtonTreeCollisionCallback; deserializeFunction : NewtonDeserialize; serializeHandle : Pointer ) : PNewtonCollision; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateTreeCollisionFromSerialization'{$ELSE}NewtonDLL{$ENDIF __GPC__};


function NewtonTreeCollisionGetFaceAtribute( const treeCollision : PNewtonCollision; const faceIndexArray : Pint ) : int; cdecl; external{$IFDEF __GPC__}name 'NewtonTreeCollisionGetFaceAtribute'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonTreeCollisionSetFaceAtribute( const treeCollision : PNewtonCollision; const faceIndexArray : Pint;
  attribute : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonTreeCollisionSetFaceAtribute'{$ELSE}NewtonDLL{$ENDIF __GPC__};

 // **********************************************************************************************
 //
 // Collision Miscelaneos function
 //
 // **********************************************************************************************
procedure NewtonReleaseCollision( const newtonWorld : PNewtonWorld; const collision : PNewtonCollision ); cdecl; external{$IFDEF __GPC__}name 'NewtonReleaseCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonCollisionCalculateAABB( const collision : PNewtonCollision; const matrix : Pfloat; p0 : Pfloat; p1 : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonCollisionCalculateAABB'{$ELSE}NewtonDLL{$ENDIF __GPC__};


 // **********************************************************************************************
 //
 // transforms utility functions
 //
 // **********************************************************************************************
procedure NewtonGetEulerAngle( const matrix : Pfloat; eulersAngles : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonGetEulerAngle'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonSetEulerAngle( const eulersAngles : Pfloat; matrix : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonSetEulerAngle'{$ELSE}NewtonDLL{$ENDIF __GPC__};

 // **********************************************************************************************
 //
 // body manipulation functions
 //
 // **********************************************************************************************
function NewtonCreateBody( const newtonWorld : PNewtonWorld; const collision : PNewtonCollision ) : PNewtonBody; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateBody'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyAddTorque( const body : PNewtonBody; const torque : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyAddTorque'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonDestroyBody( const newtonWorld : PNewtonWorld; const body : PNewtonBody ); cdecl; external{$IFDEF __GPC__}name 'NewtonDestroyBody'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyAddForce( const body : PNewtonBody; const force : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyAddForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};


procedure NewtonBodySetMatrix( const body : PNewtonBody; const matrix : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetMatrix'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetMatrixRecursive( const body : PNewtonBody; const matrix : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetMatrixRecursive'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetMassMatrix( const body : PNewtonBody; mass : float; Ixx : float; Iyy : float; Izz : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetMassMatrix'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetMaterialGroupID( const body : PNewtonBody; id : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetMaterialGroupID'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetJointRecursiveCollision( const body : PNewtonBody; state : unsigned_int ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetJointRecursiveCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetOmega( const body : PNewtonBody; const omega : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetOmega'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetVelocity( const body : PNewtonBody; const velocity : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetVelocity'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetForce( const body : PNewtonBody; const force : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetTorque( const body : PNewtonBody; const torque : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetTorque'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetLinearDamping( const body : PNewtonBody; linearDamp : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetLinearDamping'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetAngularDamping( const body : PNewtonBody; const angularDamp : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetAngularDamping'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetUserData( const body : PNewtonBody; userData : Pointer ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetUserData'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetCollision( const body : PNewtonBody; const collision : PNewtonCollision ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetAutoFreeze( const body : PNewtonBody; state : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetAutoFreeze'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetFreezeTreshold( const body : PNewtonBody; freezeSpeed2 : float; freezeOmega2 : float; framesCount : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetFreezeTreshold'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetTransformCallback( const body : PNewtonBody; callback : NewtonSetTransform ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetTransformCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetDestructorCallback( const body : PNewtonBody; callback : NewtonBodyDestructor ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetDestructorCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetAutoactiveCallback( const body : PNewtonBody; callback : NewtonBodyActivationState ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetAutoactiveCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodySetForceAndTorqueCallback( const body : PNewtonBody; callback : NewtonApplyForceAndTorque ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodySetForceAndTorqueCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonBodyGetUserData( const body : PNewtonBody ) : Pointer; cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetUserData'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonBodyGetCollision( const body : PNewtonBody ) : PNewtonCollision; cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonBodyGetMaterialGroupID( const body : PNewtonBody ) : Int; cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetMaterialGroupID'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonBodyGetJointRecursiveCollision( const body : PNewtonBody ) : Int; cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetJointRecursiveCollision'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetMatrix( const body : PNewtonBody; matrix : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetMatrix'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetMassMatrix( const body : PNewtonBody; mass : Pfloat; Ixx : Pfloat; Iyy : Pfloat; Izz : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetMassMatrix'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetInvMass( const body : PNewtonBody; invMass : Pfloat; invIxx : Pfloat; invIyy : Pfloat; invIzz : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetInvMass'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetOmega( const body : PNewtonBody; vector : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetOmega'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetVelocity( const body : PNewtonBody; vector : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetVelocity'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetForce( const body : PNewtonBody; vector : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetTorque( const body : PNewtonBody; vector : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetTorque'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonBodyGetAutoFreeze( const body : PNewtonBody ) : Int; cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetAutoFreeze'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonBodyGetLinearDamping( const body : PNewtonBody ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetLinearDamping'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetAngularDamping( const body : PNewtonBody; vector : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetAngularDamping'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetAABB( const body : PNewtonBody; p0 : Pfloat; p1 : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetAABB'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyGetFreezeTreshold( const body : PNewtonBody; freezeSpeed2 : Pfloat; freezeOmega2 : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyGetFreezeTreshold'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyAddBuoyancyForce( const body : PNewtonBody; fluidDensity : float; fluidLinearViscosity : float; fluidAngularViscosity : float;
  const gravityVector : Pfloat; buoyancyPlane : NewtonGetBuoyancyPlane; context : Pointer ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyAddBuoyancyForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBodyForEachPolygonDo( const body : PNewtonBody; callback : NewtonCollisionIterator ); cdecl; external{$IFDEF __GPC__}name 'NewtonBodyForEachPolygonDo'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonBodyRayIntersect( const body : PNewtonBody; const p0 : Pfloat; const p1 : Pfloat; normal : Pfloat ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonBodyRayIntersect'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonAddBodyImpulse( const newtonWorld : PNewtonWorld; const body : PNewtonBody;
  const pointDeltaVeloc : Pfloat; const pointPosit : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonAddBodyImpulse'{$ELSE}NewtonDLL{$ENDIF __GPC__};



 // **********************************************************************************************
 //
 // Common joint funtions
 //
 // **********************************************************************************************
function NewtonJointGetUserData( const joint : PNewtonJoint ) : Pointer; cdecl; external{$IFDEF __GPC__}name 'NewtonJointGetUserData'{$ELSE}NewtonDLL{$ENDIF __GPC__};
procedure NewtonJointSetUserData( const joint : PNewtonJoint; userData : Pointer ); cdecl; external{$IFDEF __GPC__}name 'NewtonJointSetUserData'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonJointGetCollisionState( const joint : PNewtonJoint ) : int; cdecl; external{$IFDEF __GPC__}name 'NewtonJointGetCollisionState'{$ELSE}NewtonDLL{$ENDIF __GPC__};
procedure NewtonJointSetCollisionState( const joint : PNewtonJoint; state : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonJointSetCollisionState'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonDestroyJoint( const newtonWorld : PNewtonWorld; const joint : PNewtonJoint ); cdecl; external{$IFDEF __GPC__}name 'NewtonDestroyJoint'{$ELSE}NewtonDLL{$ENDIF __GPC__};
procedure NewtonJointSetDestructor( const joint : PNewtonJoint; _destructor : NewtonConstraintDestructor ); cdecl; external{$IFDEF __GPC__}name 'NewtonJointSetDestructor'{$ELSE}NewtonDLL{$ENDIF __GPC__};

//inline NEWTON_API void NewtonSliderGetCollisionState (const NewtonJoint* joint){};
//inline NEWTON_API void NewtonSliderSetCollisionState (const NewtonJoint* joint){};
//inline NEWTON_API void NewtonHingeGetCollisionState (const NewtonJoint* joint){};
//inline NEWTON_API void NewtonHingeSetCollisionState (const NewtonJoint* joint){};
//inline NEWTON_API void NewtonBallGetCollisionState (const NewtonJoint* joint){};
//inline NEWTON_API void NewtonBallSetCollisionState (const NewtonJoint* joint){};

 // **********************************************************************************************
 //
 // Ball and Socket joint functions
 //
 // **********************************************************************************************
function NewtonConstraintCreateBall( const newtonWorld : PNewtonWorld; const pivotPoint : Pfloat;
  const childBody : PNewtonBody; const parentBody : PNewtonBody ) : PNewtonJoint; cdecl; external{$IFDEF __GPC__}name 'NewtonConstraintCreateBall'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBallSetUserCallback( const ball : PNewtonJoint; callback : NewtonBallCallBack ); cdecl; external{$IFDEF __GPC__}name 'NewtonBallSetUserCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBallGetJointAngle( const ball : PNewtonJoint; angle : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBallGetJointAngle'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBallGetJointOmega( const ball : PNewtonJoint; omega : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBallGetJointOmega'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBallGetJointForce( const ball : PNewtonJoint; force : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonBallGetJointForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonBallSetConeLimits( const ball : PNewtonJoint; const pin : Pfloat; maxConeAngle : float; maxTwistAngle : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonBallSetConeLimits'{$ELSE}NewtonDLL{$ENDIF __GPC__};

 // **********************************************************************************************
 //
 // Hinge joint functions
 //
 // **********************************************************************************************
function NewtonConstraintCreateHinge( const newtonWorld : PNewtonWorld;
  const pivotPoint : Pfloat; const pinDir : Pfloat;
  const childBody : PNewtonBody; const parentBody : PNewtonBody ) : PNewtonJoint; cdecl; external{$IFDEF __GPC__}name 'NewtonConstraintCreateHinge'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonHingeSetUserCallback( const hinge : PNewtonJoint; callback : NewtonHingeCallBack ); cdecl; external{$IFDEF __GPC__}name 'NewtonHingeSetUserCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonHingeGetJointAngle( const hinge : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonHingeGetJointAngle'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonHingeGetJointOmega( const hinge : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonHingeGetJointOmega'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonHingeGetJointForce( const hinge : PNewtonJoint; force : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonHingeGetJointForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonHingeCalculateStopAlpha( const hinge : PNewtonJoint; const desc : PNewtonHingeSliderUpdateDesc; angle : float ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonHingeCalculateStopAlpha'{$ELSE}NewtonDLL{$ENDIF __GPC__};

 // **********************************************************************************************
 //
 // Slider joint functions
 //
 // **********************************************************************************************
function NewtonConstraintCreateSlider( const newtonWorld : PNewtonWorld;
  const pivotPoint : Pfloat; const pinDir : Pfloat;
  const childBody : PNewtonBody; const parentBody : PNewtonBody ) : PNewtonJoint; cdecl; external{$IFDEF __GPC__}name 'NewtonConstraintCreateSlider'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonSliderSetUserCallback( const slider : PNewtonJoint; callback : NewtonSliderCallBack ); cdecl; external{$IFDEF __GPC__}name 'NewtonSliderSetUserCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonSliderGetJointPosit( const slider : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonSliderGetJointPosit'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonSliderGetJointVeloc( const slider : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonSliderGetJointVeloc'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonSliderGetJointForce( const slider : PNewtonJoint; force : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonSliderGetJointForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonSliderCalculateStopAccel( const slider : PNewtonJoint; const desc : PNewtonHingeSliderUpdateDesc; position : float ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonSliderCalculateStopAccel'{$ELSE}NewtonDLL{$ENDIF __GPC__};


 // **********************************************************************************************
 //
 // Corkscrew joint functions
 //
 // **********************************************************************************************
function NewtonConstraintCreateCorkscrew( const newtonWorld : PNewtonWorld;
  const pivotPoint : Pfloat; const pinDir : Pfloat;
  const childBody : PNewtonBody; const parentBody : PNewtonBody ) : PNewtonJoint; cdecl; external{$IFDEF __GPC__}name 'NewtonConstraintCreateCorkscrew'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonCorkscrewSetUserCallback( const corkscrew : PNewtonJoint; callback : NewtonCorkscrewCallBack ); cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewSetUserCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonCorkscrewGetJointPosit( const corkscrew : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewGetJointPosit'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonCorkscrewGetJointAngle( const corkscrew : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewGetJointAngle'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonCorkscrewGetJointVeloc( const corkscrew : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewGetJointVeloc'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonCorkscrewGetJointOmega( const corkscrew : PNewtonJoint ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewGetJointOmega'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonCorkscrewGetJointForce( const corkscrew : PNewtonJoint; force : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewGetJointForce'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonCorkscrewCalculateStopAlpha( const corkscrew : PNewtonJoint; const desc : PNewtonHingeSliderUpdateDesc; angle : float ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewCalculateStopAlpha'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonCorkscrewCalculateStopAccel( const corkscrew : PNewtonJoint; const desc : PNewtonHingeSliderUpdateDesc; position : float ) : float; cdecl; external{$IFDEF __GPC__}name 'NewtonCorkscrewCalculateStopAccel'{$ELSE}NewtonDLL{$ENDIF __GPC__};



  //COMMENTED OUT IN THE ORINAL C++ HEADER !

 // **********************************************************************************************
 //
 // Universal joint functions
 //
 // **********************************************************************************************
(*
 function NewtonConstraintCreateUniversal (const newtonWorld : PNewtonWorld ;
  const pivotPoint : Pfloat ; const pinDir0 : Pfloat ; const pinDir1 : Pfloat ;
  const childBody : PNewtonBody ; const parentBody : PNewtonBody ):PNewtonJoint;cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

 procedure NewtonUniversalSetUserCallback (const universal : PNewtonJoint ;callback : NewtonUniversalCallBack );cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

 function  NewtonUniversalGetJointAngle0 (const universal : PNewtonJoint ):float;cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

 function  NewtonUniversalGetJointAngle1 (const universal : PNewtonJoint):float;cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

 function  NewtonUniversalGetJointOmega0 (const universal : PNewtonJoint ): float;cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

  function  NewtonUniversalGetJointOmega1 (const universal : PNewtonJoint ):float;cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

 procedure NewtonUniversalGetJointForce (const universal : PNewtonJoint ; force : Pfloat ); cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

 function  NewtonUniversalCalculateStopAlpha0 (const universal : PNewtonJoint ; const desc : PNewtonHingeSliderUpdateDesc ; angle : float ):float;cdecl; external {$IFDEF __GPC__}name 'dWorldCreate'{$ELSE} NewtonDLL{$ENDIF __GPC__};

 function float NewtonUniversalCalculateStopAlpha1 (const NewtonJoint* universal, const NewtonHingeSliderUpdateDesc* desc, float angle);
*)
(*
 // **********************************************************************************************
 //
 // SlidingContact joint functions
 //
 // **********************************************************************************************

 NEWTON_API NewtonJoint* NewtonConstraintCreateSlidingContact (const NewtonWorld* newtonWorld,
  const float* pivotPoint, const float* pinDir0, const float* pinDir1,
  const NewtonBody* childBody, const NewtonBody* parentBody);

 NEWTON_API void NewtonSlidingContactSetUserCallback (const NewtonJoint* contact, NewtonSlidingContactCallBack callback);
*/
*)

 // **********************************************************************************************
 //
 // Up vector joint unctions
 //
 // **********************************************************************************************
function NewtonConstraintCreateUpVector( const newtonWorld : PNewtonWorld; const pinDir : Pfloat; const body : PNewtonBody ) : PNewtonJoint; cdecl; external{$IFDEF __GPC__}name 'NewtonConstraintCreateUpVector'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonUpVectorSetUserCallback( const upVector : PNewtonJoint; callback : NewtonUpVectorCallBack ); cdecl; external{$IFDEF __GPC__}name 'NewtonUpVectorSetUserCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonUpVectorGetPin( const upVector : PNewtonJoint; pin : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonUpVectorGetPin'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonUpVectorSetPin( const upVector : PNewtonJoint; const pin : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonUpVectorSetPin'{$ELSE}NewtonDLL{$ENDIF __GPC__};


 // **********************************************************************************************
 //
 // Ragdoll joint contatiner funtion
 //
 // **********************************************************************************************
function NewtonCreateRagDoll( const newtonWorld : PNewtonWorld ) : PNewtonRagDoll; cdecl; external{$IFDEF __GPC__}name 'NewtonCreateRagDoll'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonDestroyRagDoll( const newtonWorld : PNewtonWorld; const ragDoll : PNewtonRagDoll ); cdecl; external{$IFDEF __GPC__}name 'NewtonDestroyRagDoll'{$ELSE}NewtonDLL{$ENDIF __GPC__};


procedure NewtonRagDollBegin( const ragDoll : PNewtonRagDoll ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollBegin'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonRagDollEnd( const ragDoll : PNewtonRagDoll ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollEnd'{$ELSE}NewtonDLL{$ENDIF __GPC__};


//	NEWTON_API void NewtonRagDollSetFriction (const NewtonRagDoll* ragDoll, float friction);

function NewtonRagDollFindBone( const ragDoll : PNewtonRagDoll; id : int ) : PNewtonRagDollBone; cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollFindBone'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonRagDollGetRootBone( const ragDoll : PNewtonRagDoll ) : PNewtonRagDollBone; cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollGetRootBone'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonRagDollSetForceAndTorqueCallback( const ragDoll : PNewtonRagDoll; callback : NewtonApplyForceAndTorque ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollSetForceAndTorqueCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonRagDollSetTransformCallback( const ragDoll : PNewtonRagDoll; callback : NewtonSetRagDollTransform ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollSetTransformCallback'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonRagDollAddBone( const ragDoll : PNewtonRagDoll; const parent : PNewtonRagDollBone; userData : Pointer; mass : float;
  const matrix : Pfloat; const size : Pfloat ) : PNewtonRagDollBone; cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollAddBone'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonRagDollBoneGetUserData( const bone : PNewtonRagDollBone ) : Pointer; cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollBoneGetUserData'{$ELSE}NewtonDLL{$ENDIF __GPC__};

function NewtonRagDollBoneGetBody( const bone : PNewtonRagDollBone ) : PNewtonBody; cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollBoneGetBody'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonRagDollBoneSetID( const bone : PNewtonRagDollBone; id : int ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollBoneSetID'{$ELSE}NewtonDLL{$ENDIF __GPC__};


procedure NewtonRagDollBoneSetLimits( const bone : PNewtonRagDollBone;
  const coneDir : Pfloat; minConeAngle : float; maxConeAngle : float; maxTwistAngle : float;
  const bilateralConeDir : Pfloat; negativeBilateralConeAngle : float; positiveBilateralConeAngle : float ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollBoneSetLimits'{$ELSE}NewtonDLL{$ENDIF __GPC__};

//	NEWTON_API NewtonRagDollBone* NewtonRagDollBoneGetChild (const NewtonRagDollBone* bone);
//	NEWTON_API NewtonRagDollBone* NewtonRagDollBoneGetSibling (const NewtonRagDollBone* bone);
//	NEWTON_API NewtonRagDollBone* NewtonRagDollBoneGetParent (const NewtonRagDollBone* bone);
//	NEWTON_API void NewtonRagDollBoneSetLocalMatrix (const NewtonRagDollBone* bone, float* matrix);
//	NEWTON_API void NewtonRagDollBoneSetGlobalMatrix (const NewtonRagDollBone* bone, float* matrix);

procedure NewtonRagDollBoneGetLocalMatrix( const bone : PNewtonRagDollBone; matrix : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollBoneGetLocalMatrix'{$ELSE}NewtonDLL{$ENDIF __GPC__};

procedure NewtonRagDollBoneGetGlobalMatrix( const bone : PNewtonRagDollBone; matrix : Pfloat ); cdecl; external{$IFDEF __GPC__}name 'NewtonRagDollBoneGetGlobalMatrix'{$ELSE}NewtonDLL{$ENDIF __GPC__};





implementation

end.

